#include "irk.h"


#include <qpixmap.h>
#include <stdio.h>
#if !(X86)
#include "lirc_client.h"
#endif
#include <stdlib.h>
#include <unistd.h>
#include <qsocketnotifier.h>
#include <qradiobutton.h>
#include <qtimer.h>
#include <qcheckbox.h>
#include "CIRKSettingsForm.h"
#include <qpushbutton.h>
#include <qpe/config.h>
#include <qcombobox.h>
#include <qmessagebox.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <qpe/global.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <qpe/qcopenvelope_qws.h>
#include <termios.h>
#include <qlistview.h>
#include "irkimpl.h"
#include <qpe/qpeapplication.h>
#include <ctype.h>
#include <qclipboard.h>

extern const char *Infrared_xpm[];
extern const char *pocketop_xpm[];
extern const char *targus_xpm[];
extern const char *palm_xpm[];
// extern const char *ps2_xpm[];

unsigned short CGIRK::m_pocketopLookupTable[128];
unsigned short CGIRK::m_fnjpc700scancodeLookupTable[128];
unsigned short CGIRK::m_fnjpc700unicode1LookupTable[128];
unsigned short CGIRK::m_fnjpc700unicode2LookupTable[128];
/** Table of IrDA device names for various Zaurii - indexed as in the ZAURUS_MODEL enumerated type */
const char *CGIRK::m_irdaDeviceNames[ZAURUS_MODEL_MAX]={
	"/dev/ttyS2", // SL5000,SL5500
	"/dev/ttyS1", // SL5600,SLC7xx
	"/dev/rfcomm0", // Bluetooth rfcomm0
	"/dev/rfcomm1", // Bluetooth rfcomm1
	"/dev/ttyS0" // SL Serial Port
};

CGIRK::CGIRK( QWidget * parent, const char * name, WFlags f) 
 : CGIRKBase( parent, name, f )
{
	// setup a timer to cancel shift key's
	m_shiftTimer=new QTimer(this);
	connect(m_shiftTimer,SIGNAL(timeout()),SLOT(cancelShift()));
	// setup to pickup end of packet
	m_repeatTimer=new QTimer(this);
	connect(m_repeatTimer,SIGNAL(timeout()),SLOT(repeat()));
	// default modifier and caps settings
	m_capsLock=m_numLock=m_isShiftedl=m_isShiftedr=m_isNum=m_isPunct=m_isCmd=m_isCtrl=m_isAlt=false;
	m_isFnl=m_isFnr=m_isVirtualfn=false;
	// we haven't opened the keyboard yet
	m_keyboardOpen=false;
	// auto deletion in keyboard list
	m_keyboardList.setAutoDelete( TRUE );
	m_lastDown1=m_lastDown2=0;
	m_lastMod=0;
	
	// not in raw mode initially
	m_rawMode=false;
		
	// create our little status bar (show when you click on the input method icon in the toolbar)
	connect(m_on,SIGNAL(toggled(bool)),SLOT(on(bool)));
	connect(m_off,SIGNAL(toggled(bool)),SLOT(off(bool)));
	m_caps->setChecked(false);
	m_num->setChecked(false);
	connect(m_caps,SIGNAL(toggled(bool)),SLOT(caps(bool)));
	connect(m_num,SIGNAL(toggled(bool)),SLOT(num(bool)));
	connect(m_settingsButton,SIGNAL(clicked()),SLOT(settings()));
	
	// Create the settings dialog
	pSettingsForm=new CIRKSettingsForm(this,0,true);
	
	// Create the task selector
	m_pTaskSelector=new TaskSelector(this);
	// Create the task switcher
	m_pTaskSwitcher=new TaskSwitcher();

	// read the config file
	readConfig();
	
	// update our current status from the defaults in the config file
	m_on->setChecked(pSettingsForm->mDefaultOn->isChecked());
	m_off->setChecked(pSettingsForm->mDefaultOff->isChecked());

	Config config("qpe");
	config.setGroup("Screensaver");
	m_lightlevel = config.readNumEntry("Brightness",255);
	if (m_lightlevel==0) {
		m_lightlevel = 255;
		m_lighton = false;
	}
	
	// get initial rotation
	QString d = getenv("QWS_DISPLAY");
	m_rotation=0;
	if ( d.contains("Rot90") ) { // No tr
		m_rotation = 90;
	} else if ( d.contains("Rot180") ) { // No tr
		m_rotation = 180;
	} else if ( d.contains("Rot270") ) { // No tr
		m_rotation = 270;
	}
	m_isRotated=false;
}

CGIRK::~CGIRK()
{
	keyboardClose();
}

CIRKeyboard *CGIRK::currentKeyboard(int n)
{
	if(n==-1)
		return m_currentKeyboard;
	else
		return m_keyboardList.at(n);
}

void CGIRK::lircOpen()
{
#if !(X86)
	// open a socket to lircd
	fd=lirc_init("irk",1);
	if(fd!=-1)
	{
		// setup a socket notifier to tell us data is available
		sn = new QSocketNotifier( fd, QSocketNotifier::Read );
		if(sn)
		{
			connect( sn, SIGNAL(activated(int)), SLOT(dataAvailableLIRC(int)));
			m_keyboardOpen=true;
		}else{
			lircClose();
		}
	}
#endif
}

void CGIRK::lircClose()
{
#if !(X86)
	if(fd!=-1)
	{
		// close the link to lircd
		lirc_deinit();
		fd=-1;
	}
#endif
}

void CGIRK::c700Init()
{
	memset(m_fnjpc700scancodeLookupTable,0,sizeof(m_fnjpc700scancodeLookupTable));
	memset(m_fnjpc700unicode1LookupTable,0,sizeof(m_fnjpc700unicode1LookupTable));
	memset(m_fnjpc700unicode2LookupTable,0,sizeof(m_fnjpc700unicode2LookupTable));

	m_fnjpc700scancodeLookupTable['1']=0x200f;
	m_fnjpc700scancodeLookupTable['2']=0x2010;
	m_fnjpc700scancodeLookupTable['3']=0x2012;
	m_fnjpc700scancodeLookupTable['4']=0x2011;
	m_fnjpc700scancodeLookupTable['5']=0x201b;
	m_fnjpc700scancodeLookupTable['6']=0x200a;
	m_fnjpc700scancodeLookupTable['7']=0x200b;
	m_fnjpc700scancodeLookupTable['8']=0x200c;
	m_fnjpc700scancodeLookupTable['9']=0x2007;
	m_fnjpc700scancodeLookupTable['0']=0x2008;
	m_fnjpc700scancodeLookupTable['q']=0x2014;
	m_fnjpc700scancodeLookupTable['w']=Key_AsciiCircum;
	m_fnjpc700scancodeLookupTable['e']=Key_Equal;
	m_fnjpc700scancodeLookupTable['r']=Key_Plus;
	m_fnjpc700scancodeLookupTable['t']=Key_BracketLeft;
	m_fnjpc700scancodeLookupTable['y']=Key_BracketRight;
	m_fnjpc700scancodeLookupTable['u']=0xa2;
	m_fnjpc700scancodeLookupTable['i']=0xa3;
	m_fnjpc700scancodeLookupTable['o']=0x2015;
	m_fnjpc700scancodeLookupTable['p']=0x2016;
	m_fnjpc700scancodeLookupTable['a']=0x2017;
	m_fnjpc700scancodeLookupTable['s']=0x2018;
	m_fnjpc700scancodeLookupTable['d']=0xa5;
	m_fnjpc700scancodeLookupTable['f']=Key_Backslash;
	m_fnjpc700scancodeLookupTable['g']=Key_Semicolon;
	m_fnjpc700scancodeLookupTable['h']=Key_Colon;
	m_fnjpc700scancodeLookupTable['j']=Key_Asterisk;
	m_fnjpc700scancodeLookupTable['k']=0x2022;
	m_fnjpc700scancodeLookupTable['l']=Key_Bar;
	m_fnjpc700scancodeLookupTable['z']='Z';
	m_fnjpc700scancodeLookupTable['x']='X';
	m_fnjpc700scancodeLookupTable['c']='C';
	m_fnjpc700scancodeLookupTable['v']='V';
	m_fnjpc700scancodeLookupTable['b']=Key_Underscore;
	m_fnjpc700scancodeLookupTable['n']=0x2019;
	m_fnjpc700scancodeLookupTable['m']=0x201a;
	m_fnjpc700scancodeLookupTable[',']=Key_Less;
	m_fnjpc700scancodeLookupTable['.']=Key_Greater;
	m_fnjpc700scancodeLookupTable[' ']=Key_F23;
	m_fnjpc700scancodeLookupTable['-']=Key_At;
	m_fnjpc700scancodeLookupTable['!']=Key_1;
	m_fnjpc700scancodeLookupTable['@']=Key_At;
	m_fnjpc700scancodeLookupTable['#']=Key_3;
	m_fnjpc700scancodeLookupTable['$']=Key_4;
	m_fnjpc700scancodeLookupTable['%']=Key_5;
	m_fnjpc700scancodeLookupTable['^']=Qt::Key_AsciiCircum;
	m_fnjpc700scancodeLookupTable['&']=Key_6;
	m_fnjpc700scancodeLookupTable['*']=Qt::Key_Asterisk;
	m_fnjpc700scancodeLookupTable['(']=Key_8;
	m_fnjpc700scancodeLookupTable[')']=Key_9;
	m_fnjpc700scancodeLookupTable[';']=Qt::Key_Semicolon;
	m_fnjpc700scancodeLookupTable[':']=Qt::Key_Colon;
	m_fnjpc700scancodeLookupTable['\'']=Key_7;
	m_fnjpc700scancodeLookupTable['=']=Qt::Key_Equal;
	m_fnjpc700scancodeLookupTable['/']=Qt::Key_Slash;

	m_fnjpc700unicode1LookupTable['1']=0xFFFF;
	m_fnjpc700unicode1LookupTable['2']=0xFFFF;
	m_fnjpc700unicode1LookupTable['3']=0xFFFF;
	m_fnjpc700unicode1LookupTable['4']=0xFFFF;
	m_fnjpc700unicode1LookupTable['5']=0xFFFF;
	m_fnjpc700unicode1LookupTable['6']=0xFFFF;
	m_fnjpc700unicode1LookupTable['7']=0xFFFF;
	m_fnjpc700unicode1LookupTable['8']=0xFFFF;
	m_fnjpc700unicode1LookupTable['9']=0xFFFF;
	m_fnjpc700unicode1LookupTable['0']=0xFFFF;
	m_fnjpc700unicode1LookupTable['q']=0xFFFF;
	m_fnjpc700unicode1LookupTable['w']='^';
	m_fnjpc700unicode1LookupTable['e']='=';
	m_fnjpc700unicode1LookupTable['r']='+';
	m_fnjpc700unicode1LookupTable['t']='[';
	m_fnjpc700unicode1LookupTable['y']=']';
	m_fnjpc700unicode1LookupTable['u']=0xFF62;
	m_fnjpc700unicode1LookupTable['i']=0xFF63;
	m_fnjpc700unicode1LookupTable['o']=0xFFFF;
	m_fnjpc700unicode1LookupTable['p']=0xFFFF;
	m_fnjpc700unicode1LookupTable['a']=0xFFFF;
	m_fnjpc700unicode1LookupTable['s']=0xFFFF;
	m_fnjpc700unicode1LookupTable['d']=0xFF65;
	m_fnjpc700unicode1LookupTable['f']='\\';
	m_fnjpc700unicode1LookupTable['g']=';';
	m_fnjpc700unicode1LookupTable['h']=':';
	m_fnjpc700unicode1LookupTable['j']='*';
	m_fnjpc700unicode1LookupTable['k']=0xFFE5;
	m_fnjpc700unicode1LookupTable['l']='|';
	m_fnjpc700unicode1LookupTable['z']=0x1a;
	m_fnjpc700unicode1LookupTable['x']=0x18;
	m_fnjpc700unicode1LookupTable['c']=0x3;
	m_fnjpc700unicode1LookupTable['v']=0x16;
	m_fnjpc700unicode1LookupTable['b']='_';
	m_fnjpc700unicode1LookupTable['n']=0xFFFF;
	m_fnjpc700unicode1LookupTable['m']=0xFFFF;
	m_fnjpc700unicode1LookupTable[',']='<';
	m_fnjpc700unicode1LookupTable['.']='>';
	m_fnjpc700unicode1LookupTable[' ']=0xFFFF;
	m_fnjpc700unicode1LookupTable['-']='@';

	m_fnjpc700unicode2LookupTable['1']=0xFFFF;
	m_fnjpc700unicode2LookupTable['2']=0xFFFF;
	m_fnjpc700unicode2LookupTable['3']=0xFFFF;
	m_fnjpc700unicode2LookupTable['4']=0xFFFF;
	m_fnjpc700unicode2LookupTable['5']=0xFFFF;
	m_fnjpc700unicode2LookupTable['6']=0xFFFF;
	m_fnjpc700unicode2LookupTable['7']=0xFFFF;
	m_fnjpc700unicode2LookupTable['8']=0xFFFF;
	m_fnjpc700unicode2LookupTable['9']=0xFFFF;
	m_fnjpc700unicode2LookupTable['0']=0xFFFF;
	m_fnjpc700unicode2LookupTable['q']='Q'+1-'A';
	m_fnjpc700unicode2LookupTable['w']='W'+1-'A';
	m_fnjpc700unicode2LookupTable['e']='E'+1-'A';
	m_fnjpc700unicode2LookupTable['r']='R'+1-'A';
	m_fnjpc700unicode2LookupTable['t']='T'+1-'A';
	m_fnjpc700unicode2LookupTable['y']='Y'+1-'A';
	m_fnjpc700unicode2LookupTable['u']='U'+1-'A';
	m_fnjpc700unicode2LookupTable['i']='I'+1-'A';
	m_fnjpc700unicode2LookupTable['o']='O'+1-'A';
	m_fnjpc700unicode2LookupTable['p']='P'+1-'A';
	m_fnjpc700unicode2LookupTable['a']='A'+1-'A';
	m_fnjpc700unicode2LookupTable['s']='S'+1-'A';
	m_fnjpc700unicode2LookupTable['d']='D'+1-'A';
	m_fnjpc700unicode2LookupTable['f']='F'+1-'A';
	m_fnjpc700unicode2LookupTable['g']='G'+1-'A';
	m_fnjpc700unicode2LookupTable['h']='H'+1-'A';
	m_fnjpc700unicode2LookupTable['j']='J'+1-'A';
	m_fnjpc700unicode2LookupTable['k']='K'+1-'A';
	m_fnjpc700unicode2LookupTable['l']='L'+1-'A';
	m_fnjpc700unicode2LookupTable['z']='Z'+1-'A';
	m_fnjpc700unicode2LookupTable['x']='X'+1-'A';
	m_fnjpc700unicode2LookupTable['c']='C'+1-'A';
	m_fnjpc700unicode2LookupTable['v']='V'+1-'A';
	m_fnjpc700unicode2LookupTable['b']='B'+1-'A';
	m_fnjpc700unicode2LookupTable['n']='N'+1-'A';
	m_fnjpc700unicode2LookupTable['m']='M'+1-'A';
	m_fnjpc700unicode2LookupTable[',']='{';
	m_fnjpc700unicode2LookupTable['.']='}';
	m_fnjpc700unicode2LookupTable[' ']=0xFFFF;
	m_fnjpc700unicode2LookupTable['-']='@';
}

void CGIRK::targusInit()
{
	memset(m_pocketopLookupTable,0,sizeof(m_pocketopLookupTable));
	m_pocketopLookupTable[0xf]=CGIRK_KEY_TAB;
	m_pocketopLookupTable[0xd]='=';
	m_pocketopLookupTable[0x2b]='\\';
	m_pocketopLookupTable[0x35]='/';
	m_pocketopLookupTable[0x29]='`';
	m_pocketopLookupTable[0x1a]='[';
	m_pocketopLookupTable[0x1b]=']';
	m_pocketopLookupTable[0x65]=CGIRK_KEY_TODO;
	m_pocketopLookupTable[0x2]='1';
	m_pocketopLookupTable[0x3]='2';
	m_pocketopLookupTable[0x4]='3';
	m_pocketopLookupTable[0x5]='4';
	m_pocketopLookupTable[0x6]='5';
	m_pocketopLookupTable[0x7]='6';
	m_pocketopLookupTable[0x8]='7';
	m_pocketopLookupTable[0x9]='8';
	m_pocketopLookupTable[0xa]='9';
	m_pocketopLookupTable[0xb]='0';
	m_pocketopLookupTable[0x10]='q';
	m_pocketopLookupTable[0x11]='w';
	m_pocketopLookupTable[0x12]='e';
	m_pocketopLookupTable[0x13]='r';
	m_pocketopLookupTable[0x14]='t';
	m_pocketopLookupTable[0x15]='y';
	m_pocketopLookupTable[0x16]='u';
	m_pocketopLookupTable[0x17]='i';
	m_pocketopLookupTable[0x18]='o';
	m_pocketopLookupTable[0x19]='p';
	m_pocketopLookupTable[0xc]='-';
	m_pocketopLookupTable[0xe]=CGIRK_KEY_BACKSPACE;
	m_pocketopLookupTable[0x3b]=CGIRK_KEY_DATE;
	m_pocketopLookupTable[0x3c]=CGIRK_KEY_ADDRESS;
	m_pocketopLookupTable[0x3d]=CGIRK_KEY_TODO;
	m_pocketopLookupTable[0x3e]=CGIRK_KEY_MEMO;
	m_pocketopLookupTable[0x3a]=CGIRK_KEY_CAPS;
	m_pocketopLookupTable[0x1e]='a';
	m_pocketopLookupTable[0x1f]='s';
	m_pocketopLookupTable[0x20]='d';
	m_pocketopLookupTable[0x21]='f';
	m_pocketopLookupTable[0x22]='g';
	m_pocketopLookupTable[0x23]='h';
	m_pocketopLookupTable[0x24]='j';
	m_pocketopLookupTable[0x25]='k';
	m_pocketopLookupTable[0x26]='l';
	m_pocketopLookupTable[0x27]=';';
	m_pocketopLookupTable[0x28]='\'';
	m_pocketopLookupTable[0x1c]=CGIRK_KEY_RETURN;
	m_pocketopLookupTable[0x2a]=CGIRK_KEY_SHIFTL;
	m_pocketopLookupTable[0x2c]='z';
	m_pocketopLookupTable[0x2d]='x';
	m_pocketopLookupTable[0x2e]='c';
	m_pocketopLookupTable[0x2f]='v';
	m_pocketopLookupTable[0x30]='b';
	m_pocketopLookupTable[0x31]='n';
	m_pocketopLookupTable[0x32]='m';
	m_pocketopLookupTable[0x33]=',';
	m_pocketopLookupTable[0x34]='.';
	m_pocketopLookupTable[0x48]=CGIRK_KEY_UP;
	m_pocketopLookupTable[0x36]=CGIRK_KEY_SHIFTR;
	m_pocketopLookupTable[0x1d]=CGIRK_KEY_CTRL;
	// m_pocketopLookupTable[0x]=CGIRK_KEY_PUNCT;
	m_pocketopLookupTable[0x62]=CGIRK_KEY_CMD;
	// m_pocketopLookupTable[0x]=CGIRK_KEY_NUM;
	m_pocketopLookupTable[0x38]=CGIRK_KEY_ALT;
	m_pocketopLookupTable[0x39]=' ';
	m_pocketopLookupTable[0x63]=' ';
	m_pocketopLookupTable[0x68]=' ';
	m_pocketopLookupTable[0x61]=CGIRK_KEY_FUNCTION;
	m_pocketopLookupTable[0x4b]=CGIRK_KEY_LEFT;
	m_pocketopLookupTable[0x50]=CGIRK_KEY_DOWN;
	m_pocketopLookupTable[0x4d]=CGIRK_KEY_RIGHT;
	m_pocketopLookupTable[0x66]=CGIRK_KEY_DEL;
	m_pocketopLookupTable[0x1]=CGIRK_KEY_ESC;
	m_pocketopLookupTable[0x68]=CGIRK_KEY_CONFIRM;
	m_pocketopLookupTable[0x64]=CGIRK_KEY_HOME;
}

void CGIRK::pocketopInit()
{
	memset(m_pocketopLookupTable,0,sizeof(m_pocketopLookupTable));
	m_pocketopLookupTable[0x69]=CGIRK_KEY_TAB;
	m_pocketopLookupTable[0x01]='q';
	m_pocketopLookupTable[0x09]='w';
	m_pocketopLookupTable[0x11]='e';
	m_pocketopLookupTable[0x19]='r';
	m_pocketopLookupTable[0x21]='t';
	m_pocketopLookupTable[0x29]='y';
	m_pocketopLookupTable[0x31]='u';
	m_pocketopLookupTable[0x39]='i';
	m_pocketopLookupTable[0x41]='o';
	m_pocketopLookupTable[0x49]='p';
	m_pocketopLookupTable[0x51]='-';
	m_pocketopLookupTable[0x59]=CGIRK_KEY_BACKSPACE;
	m_pocketopLookupTable[0x40]=CGIRK_KEY_DATE;
	m_pocketopLookupTable[0x48]=CGIRK_KEY_ADDRESS;
	m_pocketopLookupTable[0x50]=CGIRK_KEY_TODO;
	m_pocketopLookupTable[0x58]=CGIRK_KEY_MEMO;
	m_pocketopLookupTable[0x77]=CGIRK_KEY_CAPS;
	m_pocketopLookupTable[0x07]='a';
	m_pocketopLookupTable[0x0f]='s';
	m_pocketopLookupTable[0x17]='d';
	m_pocketopLookupTable[0x1f]='f';
	m_pocketopLookupTable[0x27]='g';
	m_pocketopLookupTable[0x2f]='h';
	m_pocketopLookupTable[0x32]='j';
	m_pocketopLookupTable[0x3a]='k';
	m_pocketopLookupTable[0x42]='l';
	m_pocketopLookupTable[0x4a]=';';
	m_pocketopLookupTable[0x52]='\'';
	m_pocketopLookupTable[0x5a]=CGIRK_KEY_RETURN;
	m_pocketopLookupTable[0x65]=CGIRK_KEY_SHIFTL;
	m_pocketopLookupTable[0x03]='z';
	m_pocketopLookupTable[0x0b]='x';
	m_pocketopLookupTable[0x13]='c';
	m_pocketopLookupTable[0x1b]='v';
	m_pocketopLookupTable[0x23]='b';
	m_pocketopLookupTable[0x33]='n';
	m_pocketopLookupTable[0x3b]='m';
	m_pocketopLookupTable[0x43]=',';
	m_pocketopLookupTable[0x4b]='.';
	m_pocketopLookupTable[0x53]=CGIRK_KEY_UP;
	m_pocketopLookupTable[0x7d]=CGIRK_KEY_SHIFTR;
	m_pocketopLookupTable[0x6d]=CGIRK_KEY_CTRL;
	m_pocketopLookupTable[0x76]=CGIRK_KEY_PUNCT;
	m_pocketopLookupTable[0x75]=CGIRK_KEY_CMD;
	m_pocketopLookupTable[0x6e]=CGIRK_KEY_NUM;
	m_pocketopLookupTable[0x66]=CGIRK_KEY_ALT;
	m_pocketopLookupTable[0x2b]=' ';
	m_pocketopLookupTable[0x34]=' ';
	m_pocketopLookupTable[0x3c]=CGIRK_KEY_FUNCTION;
	m_pocketopLookupTable[0x44]=CGIRK_KEY_LEFT;
	m_pocketopLookupTable[0x4c]=CGIRK_KEY_DOWN;
	m_pocketopLookupTable[0x54]=CGIRK_KEY_RIGHT;
	m_pocketopLookupTable[0x5b]=CGIRK_KEY_DEL;
}

void CGIRK::palmInit()
{
	memset(m_pocketopLookupTable,0,sizeof(m_pocketopLookupTable));
	m_pocketopLookupTable[0x1b]=CGIRK_KEY_TAB;
	m_pocketopLookupTable[0x09]='q';
	m_pocketopLookupTable[0x0a]='w';
	m_pocketopLookupTable[0x0b]='e';
	m_pocketopLookupTable[0x0c]='r';
	m_pocketopLookupTable[0x0d]='t';
	m_pocketopLookupTable[0x0e]='y';
	m_pocketopLookupTable[0x3c]='u';
	m_pocketopLookupTable[0x3d]='i';
	m_pocketopLookupTable[0x3e]='o';
	m_pocketopLookupTable[0x3f]='p';
	m_pocketopLookupTable[0x38]='-';
	m_pocketopLookupTable[0x39]='=';
	m_pocketopLookupTable[0x32]=CGIRK_KEY_BACKSPACE;
	m_pocketopLookupTable[0x18]=CGIRK_KEY_CAPS;
	m_pocketopLookupTable[0x11]='a';
	m_pocketopLookupTable[0x12]='s';
	m_pocketopLookupTable[0x13]='d';
	m_pocketopLookupTable[0x14]='f';
	m_pocketopLookupTable[0x15]='g';
	m_pocketopLookupTable[0x16]='h';
	m_pocketopLookupTable[0x44]='j';
	m_pocketopLookupTable[0x45]='k';
	m_pocketopLookupTable[0x46]='l';
	m_pocketopLookupTable[0x47]=';';
	m_pocketopLookupTable[0x56]='\'';
	m_pocketopLookupTable[0x57]=CGIRK_KEY_RETURN;
	m_pocketopLookupTable[0x58]=CGIRK_KEY_SHIFTL;
	m_pocketopLookupTable[0x03]='z';
	m_pocketopLookupTable[0x10]='x';
	m_pocketopLookupTable[0x2c]='c';
	m_pocketopLookupTable[0x2d]='v';
	m_pocketopLookupTable[0x2e]='b';
	m_pocketopLookupTable[0x2f]='n';
	m_pocketopLookupTable[0x4c]='m';
	m_pocketopLookupTable[0x4d]=',';
	m_pocketopLookupTable[0x4e]='.';
	m_pocketopLookupTable[0x48]=CGIRK_KEY_UP;
	m_pocketopLookupTable[0x59]=CGIRK_KEY_SHIFTR;
	m_pocketopLookupTable[0x3a]='/';
	m_pocketopLookupTable[0x1a]=CGIRK_KEY_CTRL;
	m_pocketopLookupTable[0x08]=CGIRK_KEY_CMD;
	m_pocketopLookupTable[0x23]=CGIRK_KEY_ALT;
	m_pocketopLookupTable[0x22]=CGIRK_KEY_FNL;
	m_pocketopLookupTable[0x17]=' ';
	m_pocketopLookupTable[0x37]=' ';
	m_pocketopLookupTable[0x24]=CGIRK_KEY_FNR;
	m_pocketopLookupTable[0x51]=CGIRK_KEY_LEFT;
	m_pocketopLookupTable[0x52]=CGIRK_KEY_DOWN;
	m_pocketopLookupTable[0x53]=CGIRK_KEY_RIGHT;
	m_pocketopLookupTable[0x50]=CGIRK_KEY_DEL;
}

void CGIRK::TKB780UInit()
{
	memset(m_pocketopLookupTable,0,sizeof(m_pocketopLookupTable));
	m_pocketopLookupTable[0x7f]='1';
	m_pocketopLookupTable[0x7e]='2';
	m_pocketopLookupTable[0x7d]='3';
	m_pocketopLookupTable[0x7b]='4';
	m_pocketopLookupTable[0x7a]='5';
	m_pocketopLookupTable[0x79]='6';
	m_pocketopLookupTable[0x78]='7';
	m_pocketopLookupTable[0x4b]='8';
	m_pocketopLookupTable[0x4a]='9';
	m_pocketopLookupTable[0x49]='0';
	m_pocketopLookupTable[0x4f]='-';
	m_pocketopLookupTable[0x4e]='=';
	m_pocketopLookupTable[0x4d]=CGIRK_KEY_BACKSPACE;
	m_pocketopLookupTable[0x66]=CGIRK_KEY_TAB;
	m_pocketopLookupTable[0x76]='q';
	m_pocketopLookupTable[0x75]='w';
	m_pocketopLookupTable[0x74]='e';
	m_pocketopLookupTable[0x73]='r';
	m_pocketopLookupTable[0x72]='t';
	m_pocketopLookupTable[0x71]='y';
	m_pocketopLookupTable[0x43]='u';
	m_pocketopLookupTable[0x42]='i';
	m_pocketopLookupTable[0x41]='o';
	m_pocketopLookupTable[0x40]='p';
	m_pocketopLookupTable[0x47]='[';
	m_pocketopLookupTable[0x46]=']';
	m_pocketopLookupTable[0x67]=CGIRK_KEY_CAPS;
	m_pocketopLookupTable[0x6e]='a';
	m_pocketopLookupTable[0x6d]='s';
	m_pocketopLookupTable[0x6c]='d';
	m_pocketopLookupTable[0x6b]='f';
	m_pocketopLookupTable[0x6a]='g';
	m_pocketopLookupTable[0x69]='h';
	m_pocketopLookupTable[0x3b]='j';
	m_pocketopLookupTable[0x3a]='k';
	m_pocketopLookupTable[0x39]='l';
	m_pocketopLookupTable[0x38]=';';
	m_pocketopLookupTable[0x3f]='\'';
	m_pocketopLookupTable[0x3e]=CGIRK_KEY_RETURN;
	m_pocketopLookupTable[0x27]=CGIRK_KEY_SHIFTL;
	m_pocketopLookupTable[0x7c]='z';
	m_pocketopLookupTable[0x6f]='x';
	m_pocketopLookupTable[0x53]='c';
	m_pocketopLookupTable[0x52]='v';
	m_pocketopLookupTable[0x51]='b';
	m_pocketopLookupTable[0x50]='n';
	m_pocketopLookupTable[0x33]='m';
	m_pocketopLookupTable[0x32]=',';
	m_pocketopLookupTable[0x31]='.';
	m_pocketopLookupTable[0x37]='/';
	m_pocketopLookupTable[0x36]=CGIRK_KEY_UP;
	m_pocketopLookupTable[0x2f]=CGIRK_KEY_DEL;
	m_pocketopLookupTable[0x65]=CGIRK_KEY_CTRL;
	m_pocketopLookupTable[0x5d]=CGIRK_KEY_FNL;   //Fn key
	m_pocketopLookupTable[0x77]=CGIRK_KEY_CMD;
	m_pocketopLookupTable[0x5c]=CGIRK_KEY_ALT;
	m_pocketopLookupTable[0x68]=' ';
	m_pocketopLookupTable[0x48]=' ';
	m_pocketopLookupTable[0x70]=CGIRK_KEY_PUNCT;  //symbol key
	m_pocketopLookupTable[0x45]='\\';
	m_pocketopLookupTable[0x2e]=CGIRK_KEY_LEFT;
	m_pocketopLookupTable[0x2d]=CGIRK_KEY_DOWN;
	m_pocketopLookupTable[0x2c]=CGIRK_KEY_RIGHT;
	m_pocketopLookupTable[0x4c]=CGIRK_KEY_DATE;
	m_pocketopLookupTable[0x44]=CGIRK_KEY_ADDRESS;
	m_pocketopLookupTable[0x3d]=CGIRK_KEY_TODO;
	m_pocketopLookupTable[0x35]=CGIRK_KEY_MEMO;
}

void CGIRK::F8U1500Init()
{
	memset(m_pocketopLookupTable,0,sizeof(m_pocketopLookupTable));
	m_pocketopLookupTable[0x1b]='`';
	m_pocketopLookupTable[0x31]='1';
	m_pocketopLookupTable[0x32]='2';
	m_pocketopLookupTable[0x33]='3';
	m_pocketopLookupTable[0x34]='4';
	m_pocketopLookupTable[0x35]='5';
	m_pocketopLookupTable[0x36]='6';
	m_pocketopLookupTable[0x37]='7';
	m_pocketopLookupTable[0x38]='8';
	m_pocketopLookupTable[0x39]='9';
	m_pocketopLookupTable[0x30]='0';
	m_pocketopLookupTable[0x2d]='-';
	m_pocketopLookupTable[0x3d]='=';
	m_pocketopLookupTable[0x08]=CGIRK_KEY_BACKSPACE;
	m_pocketopLookupTable[0x09]=CGIRK_KEY_TAB;
	m_pocketopLookupTable[0x51]='q';
	m_pocketopLookupTable[0x57]='w';
	m_pocketopLookupTable[0x45]='e';
	m_pocketopLookupTable[0x52]='r';
	m_pocketopLookupTable[0x54]='t';
	m_pocketopLookupTable[0x59]='y';
	m_pocketopLookupTable[0x55]='u';
	m_pocketopLookupTable[0x49]='i';
	m_pocketopLookupTable[0x4f]='o';
	m_pocketopLookupTable[0x50]='p';
	m_pocketopLookupTable[0x5b]='[';
	m_pocketopLookupTable[0x5d]=']';
	m_pocketopLookupTable[0x14]=CGIRK_KEY_CAPS;
	m_pocketopLookupTable[0x41]='a';
	m_pocketopLookupTable[0x53]='s';
	m_pocketopLookupTable[0x44]='d';
	m_pocketopLookupTable[0x46]='f';
	m_pocketopLookupTable[0x47]='g';
	m_pocketopLookupTable[0x48]='h';
	m_pocketopLookupTable[0x4a]='j';
	m_pocketopLookupTable[0x4b]='k';
	m_pocketopLookupTable[0x4c]='l';
	m_pocketopLookupTable[0x3b]=';';
	m_pocketopLookupTable[0x2c]='\'';
	m_pocketopLookupTable[0x0d]=CGIRK_KEY_RETURN;
	m_pocketopLookupTable[0x10]=CGIRK_KEY_SHIFTL;
	m_pocketopLookupTable[0x5a]='z';
	m_pocketopLookupTable[0x58]='x';
	m_pocketopLookupTable[0x43]='c';
	m_pocketopLookupTable[0x56]='v';
	m_pocketopLookupTable[0x42]='b';
	m_pocketopLookupTable[0x4e]='n';
	m_pocketopLookupTable[0x4d]='m';
	m_pocketopLookupTable[0x3c]=',';
	m_pocketopLookupTable[0x2e]='.';
	m_pocketopLookupTable[0x2f]='/';
	m_pocketopLookupTable[0x26]=CGIRK_KEY_UP;
	m_pocketopLookupTable[0x12]=CGIRK_KEY_SHIFTR;
	m_pocketopLookupTable[0x11]=CGIRK_KEY_CTRL;
	m_pocketopLookupTable[0x1c]=CGIRK_KEY_CMD;
	m_pocketopLookupTable[0x1d]=CGIRK_KEY_ALT;
	m_pocketopLookupTable[0x1e]=CGIRK_KEY_PUNCT;   // Fn key
	m_pocketopLookupTable[0x20]=' ';
	m_pocketopLookupTable[0x21]=' ';
	m_pocketopLookupTable[0x5c]='\\';
	m_pocketopLookupTable[0x13]=CGIRK_KEY_DEL;
	m_pocketopLookupTable[0x27]=CGIRK_KEY_LEFT;
	m_pocketopLookupTable[0x28]=CGIRK_KEY_DOWN;
	m_pocketopLookupTable[0x25]=CGIRK_KEY_RIGHT;
	m_pocketopLookupTable[0x71]=CGIRK_KEY_DATE;
	m_pocketopLookupTable[0x72]=CGIRK_KEY_ADDRESS;
	m_pocketopLookupTable[0x73]=CGIRK_KEY_TODO;
	m_pocketopLookupTable[0x74]=CGIRK_KEY_MEMO;
}

void CGIRK::F8U1500TInit()
{
	memset(m_pocketopLookupTable,0,sizeof(m_pocketopLookupTable));
	m_pocketopLookupTable[0x1b]='`';
	m_pocketopLookupTable[0x31]='1';
	m_pocketopLookupTable[0x32]='2';
	m_pocketopLookupTable[0x33]='3';
	m_pocketopLookupTable[0x34]='4';
	m_pocketopLookupTable[0x35]='5';
	m_pocketopLookupTable[0x36]='6';
	m_pocketopLookupTable[0x37]='7';
	m_pocketopLookupTable[0x38]='8';
	m_pocketopLookupTable[0x39]='9';
	m_pocketopLookupTable[0x30]='0';
	m_pocketopLookupTable[0x2d]='-';
	m_pocketopLookupTable[0x3d]='=';
	m_pocketopLookupTable[0x08]=CGIRK_KEY_BACKSPACE;
	m_pocketopLookupTable[0x09]=CGIRK_KEY_TAB;
	m_pocketopLookupTable[0x51]='q';
	m_pocketopLookupTable[0x57]='w';
	m_pocketopLookupTable[0x45]='e';
	m_pocketopLookupTable[0x52]='r';
	m_pocketopLookupTable[0x54]='t';
	m_pocketopLookupTable[0x59]='y';
	m_pocketopLookupTable[0x55]='u';
	m_pocketopLookupTable[0x49]='i';
	m_pocketopLookupTable[0x4f]='o';
	m_pocketopLookupTable[0x50]='p';
	m_pocketopLookupTable[0x5b]='[';
	m_pocketopLookupTable[0x5d]=']';
	m_pocketopLookupTable[0x14]=CGIRK_KEY_CAPS;
	m_pocketopLookupTable[0x5e]='a';
	m_pocketopLookupTable[0x53]='s';
	m_pocketopLookupTable[0x44]='d';
	m_pocketopLookupTable[0x46]='f';
	m_pocketopLookupTable[0x47]='g';
	m_pocketopLookupTable[0x48]='h';
	m_pocketopLookupTable[0x4a]='j';
	m_pocketopLookupTable[0x4b]='k';
	m_pocketopLookupTable[0x4c]='l';
	m_pocketopLookupTable[0x3b]=';';
	m_pocketopLookupTable[0x2c]='\'';
	m_pocketopLookupTable[0x0d]=CGIRK_KEY_RETURN;
	m_pocketopLookupTable[0x10]=CGIRK_KEY_SHIFTL;
	m_pocketopLookupTable[0x5a]='z';
	m_pocketopLookupTable[0x58]='x';
	m_pocketopLookupTable[0x43]='c';
	m_pocketopLookupTable[0x56]='v';
	m_pocketopLookupTable[0x42]='b';
	m_pocketopLookupTable[0x4e]='n';
	m_pocketopLookupTable[0x4d]='m';
	m_pocketopLookupTable[0x3c]=',';
	m_pocketopLookupTable[0x2e]='.';
	m_pocketopLookupTable[0x2f]='/';
	m_pocketopLookupTable[0x26]=CGIRK_KEY_UP;
	m_pocketopLookupTable[0x12]=CGIRK_KEY_SHIFTR;
	m_pocketopLookupTable[0x11]=CGIRK_KEY_CTRL;
	m_pocketopLookupTable[0x1c]=CGIRK_KEY_CMD;
	m_pocketopLookupTable[0x1d]=CGIRK_KEY_ALT;
	m_pocketopLookupTable[0x1e]=CGIRK_KEY_PUNCT;   // Fn key
	m_pocketopLookupTable[0x20]=' ';
	m_pocketopLookupTable[0x21]=' ';
	m_pocketopLookupTable[0x5c]='\\';
	m_pocketopLookupTable[0x13]=CGIRK_KEY_DEL;
	m_pocketopLookupTable[0x27]=CGIRK_KEY_LEFT;
	m_pocketopLookupTable[0x28]=CGIRK_KEY_DOWN;
	m_pocketopLookupTable[0x25]=CGIRK_KEY_RIGHT;
	m_pocketopLookupTable[0x71]=CGIRK_KEY_DATE;
	m_pocketopLookupTable[0x72]=CGIRK_KEY_ADDRESS;
	m_pocketopLookupTable[0x73]=CGIRK_KEY_TODO;
	m_pocketopLookupTable[0x74]=CGIRK_KEY_MEMO;
}

void CGIRK::PalmOne3169WWInit()
{
	memset(m_pocketopLookupTable,0,sizeof(m_pocketopLookupTable));
	m_pocketopLookupTable[0x0e]='`';
	m_pocketopLookupTable[0x16]='1';
	m_pocketopLookupTable[0x1e]='2';
	m_pocketopLookupTable[0x26]='3';
	m_pocketopLookupTable[0x25]='4';
	m_pocketopLookupTable[0x2e]='5';
	m_pocketopLookupTable[0x36]='6';
	m_pocketopLookupTable[0x3d]='7';
	m_pocketopLookupTable[0x3e]='8';
	m_pocketopLookupTable[0x46]='9';
	m_pocketopLookupTable[0x45]='0';
	m_pocketopLookupTable[0x4e]='-';
	m_pocketopLookupTable[0x55]='=';
	m_pocketopLookupTable[0x66]=CGIRK_KEY_BACKSPACE;
	m_pocketopLookupTable[0x0d]=CGIRK_KEY_TAB;
	m_pocketopLookupTable[0x15]='q';
	m_pocketopLookupTable[0x1d]='w';
	m_pocketopLookupTable[0x24]='e';
	m_pocketopLookupTable[0x2d]='r';
	m_pocketopLookupTable[0x2c]='t';
	m_pocketopLookupTable[0x35]='y';
	m_pocketopLookupTable[0x3c]='u';
	m_pocketopLookupTable[0x43]='i';
	m_pocketopLookupTable[0x44]='o';
	m_pocketopLookupTable[0x4d]='p';
	m_pocketopLookupTable[0x54]='[';
	m_pocketopLookupTable[0x5b]=']';
	m_pocketopLookupTable[0x5d]='\\';
	m_pocketopLookupTable[0x58]=CGIRK_KEY_CAPS;
	m_pocketopLookupTable[0x1c]='a';
	m_pocketopLookupTable[0x1b]='s';
	m_pocketopLookupTable[0x23]='d';
	m_pocketopLookupTable[0x2b]='f';
	m_pocketopLookupTable[0x34]='g';
	m_pocketopLookupTable[0x33]='h';
	m_pocketopLookupTable[0x3b]='j';
	m_pocketopLookupTable[0x42]='k';
	m_pocketopLookupTable[0x4b]='l';
	m_pocketopLookupTable[0x4c]=';';
	m_pocketopLookupTable[0x52]='\'';
	m_pocketopLookupTable[0x5a]=CGIRK_KEY_RETURN;
	m_pocketopLookupTable[0x12]=CGIRK_KEY_SHIFTL;
	m_pocketopLookupTable[0x1a]='z';
	m_pocketopLookupTable[0x22]='x';
	m_pocketopLookupTable[0x21]='c';
	m_pocketopLookupTable[0x2a]='v';
	m_pocketopLookupTable[0x32]='b';
	m_pocketopLookupTable[0x31]='n';
	m_pocketopLookupTable[0x3a]='m';
	m_pocketopLookupTable[0x48]=',';
	m_pocketopLookupTable[0x49]='.';
	m_pocketopLookupTable[0x28]=CGIRK_KEY_UP;
	m_pocketopLookupTable[0x59]=CGIRK_KEY_SHIFTR;
	m_pocketopLookupTable[0x4a]='/';
	m_pocketopLookupTable[0x14]=CGIRK_KEY_CTRL;
	m_pocketopLookupTable[0x02]=CGIRK_KEY_FNL;   // Fn key
	m_pocketopLookupTable[0x03]=CGIRK_KEY_CMD;   // Win key
	m_pocketopLookupTable[0x11]=CGIRK_KEY_FNR;   // IE key
	m_pocketopLookupTable[0x29]=' ';
	m_pocketopLookupTable[0x30]=CGIRK_KEY_ALT;
	m_pocketopLookupTable[0x5e]=CGIRK_KEY_LEFT;
	m_pocketopLookupTable[0x60]=CGIRK_KEY_DOWN;
	m_pocketopLookupTable[0x2f]=CGIRK_KEY_RIGHT;
	m_pocketopLookupTable[0x1f]=CGIRK_KEY_DEL;
}

void CGIRK::HPfoldableBTInit()
{
	memset(m_pocketopLookupTable,0,sizeof(m_pocketopLookupTable));
	m_pocketopLookupTable[0x08]=CGIRK_KEY_ESC;
	m_pocketopLookupTable[0x0e]='`';
	m_pocketopLookupTable[0x16]='1';
	m_pocketopLookupTable[0x1e]='2';
	m_pocketopLookupTable[0x26]='3';
	m_pocketopLookupTable[0x25]='4';
	m_pocketopLookupTable[0x2e]='5';
	m_pocketopLookupTable[0x36]='6';
	m_pocketopLookupTable[0x3d]='7';
	m_pocketopLookupTable[0x3e]='8';
	m_pocketopLookupTable[0x46]='9';
	m_pocketopLookupTable[0x45]='0';
	m_pocketopLookupTable[0x4e]='-';
	m_pocketopLookupTable[0x55]='=';
	m_pocketopLookupTable[0x71]=CGIRK_KEY_BACKSPACE;
	m_pocketopLookupTable[0x0d]=CGIRK_KEY_TAB;
	m_pocketopLookupTable[0x15]='q';
	m_pocketopLookupTable[0x1d]='w';
	m_pocketopLookupTable[0x24]='e';
	m_pocketopLookupTable[0x2d]='r';
	m_pocketopLookupTable[0x2c]='t';
	m_pocketopLookupTable[0x35]='y';
	m_pocketopLookupTable[0x3c]='u';
	m_pocketopLookupTable[0x43]='i';
	m_pocketopLookupTable[0x44]='o';
	m_pocketopLookupTable[0x4d]='p';
	m_pocketopLookupTable[0x54]='[';
	m_pocketopLookupTable[0x5b]=']';
	m_pocketopLookupTable[0x5d]='\\';
	m_pocketopLookupTable[0x58]=CGIRK_KEY_CAPS;
	m_pocketopLookupTable[0x1c]='a';
	m_pocketopLookupTable[0x1b]='s';
	m_pocketopLookupTable[0x23]='d';
	m_pocketopLookupTable[0x2b]='f';
	m_pocketopLookupTable[0x34]='g';
	m_pocketopLookupTable[0x33]='h';
	m_pocketopLookupTable[0x3b]='j';
	m_pocketopLookupTable[0x42]='k';
	m_pocketopLookupTable[0x4b]='l';
	m_pocketopLookupTable[0x4c]=';';
	m_pocketopLookupTable[0x52]='\'';
	m_pocketopLookupTable[0x5a]=CGIRK_KEY_RETURN;
	m_pocketopLookupTable[0x12]=CGIRK_KEY_SHIFTL;
	m_pocketopLookupTable[0x1a]='z';
	m_pocketopLookupTable[0x22]='x';
	m_pocketopLookupTable[0x21]='c';
	m_pocketopLookupTable[0x2a]='v';
	m_pocketopLookupTable[0x32]='b';
	m_pocketopLookupTable[0x31]='n';
	m_pocketopLookupTable[0x3a]='m';
	m_pocketopLookupTable[0x41]=',';
	m_pocketopLookupTable[0x49]='.';
	m_pocketopLookupTable[0x28]=CGIRK_KEY_UP;
	m_pocketopLookupTable[0x59]=CGIRK_KEY_SHIFTR;
	m_pocketopLookupTable[0x4a]='/';
	m_pocketopLookupTable[0x14]=CGIRK_KEY_CTRL;
	m_pocketopLookupTable[0x02]=CGIRK_KEY_FNL;   // Fn key
	m_pocketopLookupTable[0x07]=CGIRK_KEY_CMD;   // Win key
	m_pocketopLookupTable[0x10]=CGIRK_KEY_ALT;
	m_pocketopLookupTable[0x5c]=' ';
	m_pocketopLookupTable[0x13]=CGIRK_KEY_FNR;   // AltGr key
	m_pocketopLookupTable[0x5e]=CGIRK_KEY_LEFT;
	m_pocketopLookupTable[0x60]=CGIRK_KEY_DOWN;
	m_pocketopLookupTable[0x2f]=CGIRK_KEY_RIGHT;
	m_pocketopLookupTable[0x66]=CGIRK_KEY_DEL;
}

void CGIRK::patapataInit()
{
	memset(m_pocketopLookupTable,0,sizeof(m_pocketopLookupTable));
	m_pocketopLookupTable[0x0]='1';
	m_pocketopLookupTable[0x1]='2';
	m_pocketopLookupTable[0x2]='3';
	m_pocketopLookupTable[0x4]='4';
	m_pocketopLookupTable[0x5]='5';
	m_pocketopLookupTable[0x6]='6';
	m_pocketopLookupTable[0x7]='7';
	m_pocketopLookupTable[0x34]='8';
	m_pocketopLookupTable[0x35]='9';
	m_pocketopLookupTable[0x36]='0';
	m_pocketopLookupTable[0x9]='q';
	m_pocketopLookupTable[0xa]='w';
	m_pocketopLookupTable[0xb]='e';
	m_pocketopLookupTable[0xc]='r';
	m_pocketopLookupTable[0xd]='t';
	m_pocketopLookupTable[0xe]='y';
	m_pocketopLookupTable[0x3c]='u';
	m_pocketopLookupTable[0x3d]='i';
	m_pocketopLookupTable[0x3e]='o';
	m_pocketopLookupTable[0x3f]='p';
	m_pocketopLookupTable[0x11]='a';
	m_pocketopLookupTable[0x12]='s';
	m_pocketopLookupTable[0x13]='d';
	m_pocketopLookupTable[0x14]='f';
	m_pocketopLookupTable[0x15]='g';
	m_pocketopLookupTable[0x16]='h';
	m_pocketopLookupTable[0x44]='j';
	m_pocketopLookupTable[0x45]='k';
	m_pocketopLookupTable[0x46]='l';
	m_pocketopLookupTable[0x3]='z';
	m_pocketopLookupTable[0x10]='x';
	m_pocketopLookupTable[0x2c]='c';
	m_pocketopLookupTable[0x2d]='v';
	m_pocketopLookupTable[0x2e]='b';
	m_pocketopLookupTable[0x2f]='n';
	m_pocketopLookupTable[0x4c]='m';
	m_pocketopLookupTable[0x19]=CGIRK_KEY_TAB;
	m_pocketopLookupTable[0x4d]=',';
	m_pocketopLookupTable[0x4e]='.';
	m_pocketopLookupTable[0x47]=';';
	m_pocketopLookupTable[0x58]=CGIRK_KEY_SHIFTL;
	m_pocketopLookupTable[0x59]=CGIRK_KEY_SHIFTR;
	m_pocketopLookupTable[0x41]=CGIRK_KEY_RETURN;
	m_pocketopLookupTable[0x38]='[';
	m_pocketopLookupTable[0x39]=']';
	m_pocketopLookupTable[0x3a]='\\';
	m_pocketopLookupTable[0x32]=CGIRK_KEY_BACKSPACE;
	m_pocketopLookupTable[0x30]='-';
	m_pocketopLookupTable[0x49]=CGIRK_KEY_UP;
	m_pocketopLookupTable[0x52]=CGIRK_KEY_DOWN;
	m_pocketopLookupTable[0x51]=CGIRK_KEY_LEFT;
	m_pocketopLookupTable[0x53]=CGIRK_KEY_RIGHT;
	m_pocketopLookupTable[0x18]=CGIRK_KEY_CAPS;
	m_pocketopLookupTable[0x1a]=CGIRK_KEY_CTRL;
	m_pocketopLookupTable[0x33]=CGIRK_KEY_DATE;
	m_pocketopLookupTable[0x3b]=CGIRK_KEY_ADDRESS;
	m_pocketopLookupTable[0x42]=CGIRK_KEY_TODO;
	m_pocketopLookupTable[0x4a]=CGIRK_KEY_MEMO;
	m_pocketopLookupTable[0x17]=' ';
	m_pocketopLookupTable[0x50]=CGIRK_KEY_DEL;
	m_pocketopLookupTable[0x4f]=CGIRK_KEY_ESC;
	m_pocketopLookupTable[0x40]='\'';
	m_pocketopLookupTable[0x48]='/';
	m_pocketopLookupTable[0x31]='=';
	m_pocketopLookupTable[0xf]='`';
	m_pocketopLookupTable[0x37]=4164; //CGIRK_ZENHAN;
	m_pocketopLookupTable[0x22]=CGIRK_KEY_PUNCT;
	m_pocketopLookupTable[0x23]=CGIRK_KEY_ALT;
	m_pocketopLookupTable[0x8]=CGIRK_KEY_CMD;

	m_pocketopLookupTable[0x60]=CGIRK_KEY_UP;
	m_pocketopLookupTable[0x61]=CGIRK_KEY_DOWN;
	m_pocketopLookupTable[0x62]=CGIRK_KEY_LEFT;
	m_pocketopLookupTable[0x63]=CGIRK_KEY_RIGHT;
	m_pocketopLookupTable[0x64]=' ';
	m_pocketopLookupTable[0x65]=CGIRK_KEY_RETURN;

	m_pocketopLookupTable[0x68]='d';
	m_pocketopLookupTable[0x69]='c';

	m_pocketopLookupTable[0x6b]='s';
	m_pocketopLookupTable[0x6c]='x';
	m_pocketopLookupTable[0x6d]='a';
	m_pocketopLookupTable[0x6e]='z';
}

void CGIRK::US102Init()
{
	memset(m_pocketopLookupTable,0,sizeof(m_pocketopLookupTable));
	m_pocketopLookupTable[0x0d]=CGIRK_KEY_TAB;
	m_pocketopLookupTable[0x0e]='`';
	m_pocketopLookupTable[0x11]=CGIRK_KEY_ALT;
	m_pocketopLookupTable[0x12]=CGIRK_KEY_SHIFTL;
	m_pocketopLookupTable[0x14]=CGIRK_KEY_CTRL;
	m_pocketopLookupTable[0x15]='q';
	m_pocketopLookupTable[0x16]='1';
	m_pocketopLookupTable[0x1a]='z';
	m_pocketopLookupTable[0x1b]='s';
	m_pocketopLookupTable[0x1c]='a';
	m_pocketopLookupTable[0x1d]='w';
	m_pocketopLookupTable[0x1e]='2';
	m_pocketopLookupTable[0x1F]=CGIRK_KEY_CMD;   // WinL: E0 1F
	m_pocketopLookupTable[0x21]='c';
	m_pocketopLookupTable[0x22]='x';
	m_pocketopLookupTable[0x23]='d';
	m_pocketopLookupTable[0x24]='e';
	m_pocketopLookupTable[0x25]='4';
	m_pocketopLookupTable[0x26]='3';
	m_pocketopLookupTable[0x29]=' ';
	m_pocketopLookupTable[0x2a]='v';
	m_pocketopLookupTable[0x2c]='t';
	m_pocketopLookupTable[0x2b]='f';
	m_pocketopLookupTable[0x2d]='r';
	m_pocketopLookupTable[0x2e]='5';
	m_pocketopLookupTable[0x31]='n';
	m_pocketopLookupTable[0x32]='b';
	m_pocketopLookupTable[0x33]='h';
	m_pocketopLookupTable[0x34]='g';
	m_pocketopLookupTable[0x35]='y';
	m_pocketopLookupTable[0x36]='6';
	m_pocketopLookupTable[0x3a]='m';
	m_pocketopLookupTable[0x3b]='j';
	m_pocketopLookupTable[0x3c]='u';
	m_pocketopLookupTable[0x3d]='7';
	m_pocketopLookupTable[0x3e]='8';
	m_pocketopLookupTable[0x41]=',';
	m_pocketopLookupTable[0x42]='k';
	m_pocketopLookupTable[0x43]='i';
	m_pocketopLookupTable[0x44]='o';
	m_pocketopLookupTable[0x45]='0';
	m_pocketopLookupTable[0x46]='9';
	m_pocketopLookupTable[0x49]='.';
	m_pocketopLookupTable[0x4a]='/';
	m_pocketopLookupTable[0x4b]='l';
	m_pocketopLookupTable[0x4c]=';';
	m_pocketopLookupTable[0x4d]='p';
	m_pocketopLookupTable[0x4e]='-';
	m_pocketopLookupTable[0x52]='\'';
	m_pocketopLookupTable[0x54]='[';
	m_pocketopLookupTable[0x55]='=';
	m_pocketopLookupTable[0x58]=CGIRK_KEY_CAPS;
	m_pocketopLookupTable[0x59]=CGIRK_KEY_SHIFTR;
	m_pocketopLookupTable[0x5a]=CGIRK_KEY_RETURN;
	m_pocketopLookupTable[0x5b]=']';
	m_pocketopLookupTable[0x5d]='\\';
	m_pocketopLookupTable[0x66]=CGIRK_KEY_BACKSPACE;
	m_pocketopLookupTable[0x6B]=CGIRK_KEY_LEFT;  // E0 6B
	m_pocketopLookupTable[0x71]=CGIRK_KEY_DEL;   // E0 71
	m_pocketopLookupTable[0x72]=CGIRK_KEY_DOWN;  // E0 72
	m_pocketopLookupTable[0x74]=CGIRK_KEY_RIGHT; // E0 74
	m_pocketopLookupTable[0x75]=CGIRK_KEY_UP;    // E0 75
	m_pocketopLookupTable[0x76]=CGIRK_KEY_ESC;

//	m_pocketopLookupTable[0x13]=CGIRK_KEY_FNR;   // AltGr key ?? anoda
//	m_pocketopLookupTable[0x02]=CGIRK_KEY_FNL;   // Fn key ?? anoda
}

void CGIRK::JP106Init()
{
	memset(m_pocketopLookupTable,0,sizeof(m_pocketopLookupTable));
	m_pocketopLookupTable[0x0d]=CGIRK_KEY_TAB;
	m_pocketopLookupTable[0x11]=CGIRK_KEY_ALT;
	m_pocketopLookupTable[0x12]=CGIRK_KEY_SHIFTL;
	m_pocketopLookupTable[0x14]=CGIRK_KEY_CTRL;
	m_pocketopLookupTable[0x15]='q';
	m_pocketopLookupTable[0x16]='1';
	m_pocketopLookupTable[0x1a]='z';
	m_pocketopLookupTable[0x1b]='s';
	m_pocketopLookupTable[0x1c]='a';
	m_pocketopLookupTable[0x1d]='w';
	m_pocketopLookupTable[0x1e]='2';
	m_pocketopLookupTable[0x1F]=CGIRK_KEY_CMD;   // WinL: E0 1F
	m_pocketopLookupTable[0x21]='c';
	m_pocketopLookupTable[0x22]='x';
	m_pocketopLookupTable[0x23]='d';
	m_pocketopLookupTable[0x24]='e';
	m_pocketopLookupTable[0x25]='4';
	m_pocketopLookupTable[0x26]='3';
	m_pocketopLookupTable[0x29]=' ';
	m_pocketopLookupTable[0x2a]='v';
	m_pocketopLookupTable[0x2c]='t';
	m_pocketopLookupTable[0x2b]='f';
	m_pocketopLookupTable[0x2d]='r';
	m_pocketopLookupTable[0x2e]='5';
	m_pocketopLookupTable[0x31]='n';
	m_pocketopLookupTable[0x32]='b';
	m_pocketopLookupTable[0x33]='h';
	m_pocketopLookupTable[0x34]='g';
	m_pocketopLookupTable[0x35]='y';
	m_pocketopLookupTable[0x36]='6';
	m_pocketopLookupTable[0x3a]='m';
	m_pocketopLookupTable[0x3b]='j';
	m_pocketopLookupTable[0x3c]='u';
	m_pocketopLookupTable[0x3d]='7';
	m_pocketopLookupTable[0x3e]='8';
	m_pocketopLookupTable[0x41]=',';
	m_pocketopLookupTable[0x42]='k';
	m_pocketopLookupTable[0x43]='i';
	m_pocketopLookupTable[0x44]='o';
	m_pocketopLookupTable[0x45]='0';
	m_pocketopLookupTable[0x46]='9';
	m_pocketopLookupTable[0x49]='.';
	m_pocketopLookupTable[0x4a]='/';
	m_pocketopLookupTable[0x4b]='l';
	m_pocketopLookupTable[0x4c]=';';
	m_pocketopLookupTable[0x4d]='p';
	m_pocketopLookupTable[0x4e]='-';
	m_pocketopLookupTable[0x51]='\\';
	m_pocketopLookupTable[0x52]=':';
	m_pocketopLookupTable[0x54]='@';
	m_pocketopLookupTable[0x55]='^';
	m_pocketopLookupTable[0x58]=CGIRK_KEY_CAPS;
	m_pocketopLookupTable[0x59]=CGIRK_KEY_SHIFTR;
	m_pocketopLookupTable[0x5a]=CGIRK_KEY_RETURN;
	m_pocketopLookupTable[0x5b]='[';
	m_pocketopLookupTable[0x5d]=']';
	m_pocketopLookupTable[0x66]=CGIRK_KEY_BACKSPACE;
	m_pocketopLookupTable[0x6A]='\\';
	m_pocketopLookupTable[0x6B]=CGIRK_KEY_LEFT;  // E0 6B
	m_pocketopLookupTable[0x71]=CGIRK_KEY_DEL;   // E0 71
	m_pocketopLookupTable[0x72]=CGIRK_KEY_DOWN;  // E0 72
	m_pocketopLookupTable[0x74]=CGIRK_KEY_RIGHT; // E0 74
	m_pocketopLookupTable[0x75]=CGIRK_KEY_UP;    // E0 75
	m_pocketopLookupTable[0x76]=CGIRK_KEY_ESC;

//	m_pocketopLookupTable[0x13]=CGIRK_KEY_FNR;   // AltGr key ?? anoda
//	m_pocketopLookupTable[0x02]=CGIRK_KEY_FNL;   // Fn key ?? anoda
}

void CGIRK::palmOpen()
{
	// Init the pocketop lookup table
	palmInit();
	c700Init();
#if !(X86)
	// open the IRDA serial port
	fd=open(m_irdaDeviceNames[m_zaurusModel],O_RDONLY|O_NONBLOCK|O_NOCTTY);
	if(fd!=-1)
	{
		struct termios options;
		// Setup the IR serial port
		tcgetattr(fd, &options);
		
		options.c_iflag = IGNBRK;
		options.c_cflag = (CS8|CREAD|CLOCAL|CRTSCTS);
		options.c_oflag = (NL0|CR0|TAB0|BS0|VT0|FF0);
		options.c_lflag = 0;
		
		// 9600 baud
		cfsetispeed(&options, B9600);
		cfsetospeed(&options, B9600);
		
		// Set the new options for the port...
		tcsetattr(fd, TCSANOW, &options);
	
		// setup a socket notifier to tell us data is available
		sn = new QSocketNotifier( fd, QSocketNotifier::Read);
		if(sn)
		{
			connect( sn, SIGNAL(activated(int)), SLOT(dataAvailablePalm(int)));
			m_keyboardOpen=true;
		}else{
			::close(fd);
			fd=-1;
		}
	}else{
		QMessageBox mb("IRK","Couldn't open IrDA Serial Port.\nEither something else is using the Infrared, or you're using an OpenZaurus ROM with broken IrDA serial support.",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		mb.setIconPixmap(QPixmap((const char **)palm_xpm));
		mb.exec();
		m_on->setChecked(false);
	}
#endif
}


void CGIRK::targusOpen()
{
	// Init the pocketop lookup table
	targusInit();
	c700Init();
#if !(X86)
	// open the IRDA serial port
	fd=open(m_irdaDeviceNames[m_zaurusModel],O_RDONLY|O_NONBLOCK|O_NOCTTY);
	if(fd!=-1)
	{
		struct termios options;
		// Setup the IR serial port
		tcgetattr(fd, &options);
		
		options.c_iflag = IGNBRK;
		options.c_cflag = (CS8|CREAD|CLOCAL|CRTSCTS);
		options.c_oflag = (NL0|CR0|TAB0|BS0|VT0|FF0);
		options.c_lflag = 0;
		
		// 9600 baud
		cfsetispeed(&options, B9600);
		cfsetospeed(&options, B9600);
		
		// Set the new options for the port...
		tcsetattr(fd, TCSANOW, &options);
	
		// setup a socket notifier to tell us data is available
		sn = new QSocketNotifier( fd, QSocketNotifier::Read);
		if(sn)
		{
			connect( sn, SIGNAL(activated(int)), SLOT(dataAvailablePocketop(int)));
			m_keyboardOpen=true;
		}else{
			::close(fd);
			fd=-1;
		}
	}else{
		QMessageBox mb("IRK","Couldn't open IrDA Serial Port.\nEither something else is using the Infrared, or you're using an OpenZaurus ROM with broken IrDA serial support.",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		mb.setIconPixmap(QPixmap((const char **)targus_xpm));
		mb.exec();
		m_on->setChecked(false);
	}
#endif
}


void CGIRK::pocketopOpen()
{
	// Init the pocketop lookup table
	pocketopInit();
	c700Init();
#if !(X86)
	// open the IRDA serial port
	fd=open(m_irdaDeviceNames[m_zaurusModel],O_RDONLY|O_NONBLOCK|O_NOCTTY);
	if(fd!=-1)
	{
		struct termios options;
		// Setup the IR serial port
		tcgetattr(fd, &options);
		
		options.c_iflag = IGNBRK;
		options.c_cflag = (CS8|CREAD|CLOCAL|CRTSCTS);
		options.c_oflag = (NL0|CR0|TAB0|BS0|VT0|FF0);
		options.c_lflag = 0;
		
		// 9600 baud
		cfsetispeed(&options, B9600);
		cfsetospeed(&options, B9600);
		
		// Set the new options for the port...
		tcsetattr(fd, TCSANOW, &options);
	
		// setup a socket notifier to tell us data is available
		sn = new QSocketNotifier( fd, QSocketNotifier::Read);
		if(sn)
		{
			connect( sn, SIGNAL(activated(int)), SLOT(dataAvailablePocketop(int)));
			m_keyboardOpen=true;
		}else{
			::close(fd);
			fd=-1;
		}
	}else{
		QMessageBox mb("IRK","Couldn't open IrDA Serial Port.\nEither something else is using the Infrared, or you're using an OpenZaurus ROM with broken IrDA serial support.",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		mb.setIconPixmap(QPixmap((const char **)pocketop_xpm));
		mb.exec();
		m_on->setChecked(false);
	}
#endif
}

void CGIRK::TKB780UOpen()
{
	// Init the pocketop lookup table
	TKB780UInit();
	c700Init();
#if !(X86)
	// open the IRDA serial port
	fd=open(m_irdaDeviceNames[m_zaurusModel],O_RDONLY|O_NONBLOCK|O_NOCTTY);
	if(fd!=-1)
	{
		struct termios options;
		// Setup the IR serial port
		tcgetattr(fd, &options);
		
		options.c_iflag = IGNBRK;
		options.c_cflag = (CS8|CREAD|CLOCAL|CRTSCTS);
		options.c_oflag = (NL0|CR0|TAB0|BS0|VT0|FF0);
		options.c_lflag = 0;
		
		// 9600 baud
		cfsetispeed(&options, B9600);
		cfsetospeed(&options, B9600);
		
		// Set the new options for the port...
		tcsetattr(fd, TCSANOW, &options);
	
		// setup a socket notifier to tell us data is available
		sn = new QSocketNotifier( fd, QSocketNotifier::Read);
		if(sn)
		{
			connect( sn, SIGNAL(activated(int)), SLOT(dataAvailableTKB780U(int)));
			m_keyboardOpen=true;
		}else{
			::close(fd);
			fd=-1;
		}
	}else{
		QMessageBox mb("IRK","Couldn't open IrDA Serial Port.\nEither something else is using the Infrared, or you're using an OpenZaurus ROM with broken IrDA serial support.",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		mb.setIconPixmap(QPixmap((const char **)palm_xpm));
		mb.exec();
		m_on->setChecked(false);
	}
#endif
}

void CGIRK::F8U1500Open()
{
	// Init the pocketop lookup table
	F8U1500Init();
	c700Init();
#if !(X86)
	// open the IRDA serial port
	fd=open(m_irdaDeviceNames[m_zaurusModel],O_RDONLY|O_NONBLOCK|O_NOCTTY);
	if(fd!=-1)
	{
		struct termios options;
		// Setup the IR serial port
		tcgetattr(fd, &options);
		
		options.c_iflag = IGNBRK;
		options.c_cflag = (CS8|CREAD|CLOCAL|CRTSCTS);
		options.c_oflag = (NL0|CR0|TAB0|BS0|VT0|FF0);
		options.c_lflag = 0;
		
		// 9600 baud
		cfsetispeed(&options, B9600);
		cfsetospeed(&options, B9600);
		
		// Set the new options for the port...
		tcsetattr(fd, TCSANOW, &options);
	
		// setup a socket notifier to tell us data is available
		sn = new QSocketNotifier( fd, QSocketNotifier::Read);
		if(sn)
		{
			connect( sn, SIGNAL(activated(int)), SLOT(dataAvailableF8U1500(int)));
			m_keyboardOpen=true;
		}else{
			::close(fd);
			fd=-1;
		}
	}else{
		QMessageBox mb("IRK","Couldn't open IrDA Serial Port.\nEither something else is using the Infrared, or you're using an OpenZaurus ROM with broken IrDA serial support.",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		mb.setIconPixmap(QPixmap((const char **)palm_xpm));
		mb.exec();
		m_on->setChecked(false);
	}
#endif
}

void CGIRK::F8U1500TOpen()
{
	// Init the pocketop lookup table
	F8U1500TInit();
	c700Init();
#if !(X86)
	// open the IRDA serial port
	fd=open(m_irdaDeviceNames[m_zaurusModel],O_RDONLY|O_NONBLOCK|O_NOCTTY);
	if(fd!=-1)
	{
		struct termios options;
		// Setup the IR serial port
		tcgetattr(fd, &options);
		
		options.c_iflag = IGNBRK;
		options.c_cflag = (CS8|CREAD|CLOCAL|CRTSCTS);
		options.c_oflag = (NL0|CR0|TAB0|BS0|VT0|FF0);
		options.c_lflag = 0;
		
		// 9600 baud
		cfsetispeed(&options, B9600);
		cfsetospeed(&options, B9600);
		
		// Set the new options for the port...
		tcsetattr(fd, TCSANOW, &options);
	
		// setup a socket notifier to tell us data is available
		sn = new QSocketNotifier( fd, QSocketNotifier::Read);
		if(sn)
		{
			connect( sn, SIGNAL(activated(int)), SLOT(dataAvailableF8U1500T(int)));
			m_keyboardOpen=true;
		}else{
			::close(fd);
			fd=-1;
		}
	}else{
		QMessageBox mb("IRK","Couldn't open IrDA Serial Port.\nEither something else is using the Infrared, or you're using an OpenZaurus ROM with broken IrDA serial support.",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		mb.setIconPixmap(QPixmap((const char **)palm_xpm));
		mb.exec();
		m_on->setChecked(false);
	}
#endif
}

void CGIRK::PalmOne3169WWOpen()
{
	// Init the pocketop lookup table
	PalmOne3169WWInit();
	c700Init();
#if !(X86)
	// open the IRDA serial port
	fd=open(m_irdaDeviceNames[m_zaurusModel],O_RDONLY|O_NONBLOCK|O_NOCTTY);
	if(fd!=-1)
	{
		struct termios options;
		// Setup the IR serial port
		tcgetattr(fd, &options);
		
		options.c_iflag = IGNBRK;
		options.c_cflag = (CS8|CREAD|CLOCAL|CRTSCTS);
		options.c_oflag = (NL0|CR0|TAB0|BS0|VT0|FF0);
		options.c_lflag = 0;
		
		// 9600 baud
		cfsetispeed(&options, B9600);
		cfsetospeed(&options, B9600);
		
		// Set the new options for the port...
		tcsetattr(fd, TCSANOW, &options);
	
		// setup a socket notifier to tell us data is available
		sn = new QSocketNotifier( fd, QSocketNotifier::Read);
		if(sn)
		{
			connect( sn, SIGNAL(activated(int)), SLOT(dataAvailablePalmOne3169WW(int)));
			m_keyboardOpen=true;
		}else{
			::close(fd);
			fd=-1;
		}
	}else{
		QMessageBox mb("IRK","Couldn't open IrDA Serial Port.\nEither something else is using the Infrared, or you're using an OpenZaurus ROM with broken IrDA serial support.",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		mb.setIconPixmap(QPixmap((const char **)palm_xpm));
		mb.exec();
		m_on->setChecked(false);
	}
#endif
}

void CGIRK::HPfoldableBTOpen()
{
	// Init the pocketop lookup table
	HPfoldableBTInit();
	c700Init();
#if !(X86)
	// open the IRDA serial port
	fd=open(m_irdaDeviceNames[m_zaurusModel],O_RDONLY|O_NONBLOCK|O_NOCTTY);
	if(fd!=-1)
	{
		struct termios options;
		// Setup the Bluetooth serial port
		tcgetattr(fd, &options);
		
		options.c_iflag = IGNBRK;
		options.c_cflag = (CS8|CREAD|CLOCAL|CRTSCTS);
		options.c_oflag = (NL0|CR0|TAB0|BS0|VT0|FF0);
		options.c_lflag = 0;
		
		// 9600 baud
		cfsetispeed(&options, B9600);
		cfsetospeed(&options, B9600);
		
		// Set the new options for the port...
		tcsetattr(fd, TCSANOW, &options);
	
		// setup a socket notifier to tell us data is available
		sn = new QSocketNotifier( fd, QSocketNotifier::Read);
		if(sn)
		{
			connect( sn, SIGNAL(activated(int)), SLOT(dataAvailableHPfoldableBT(int)));
			m_keyboardOpen=true;
		}else{
			::close(fd);
			fd=-1;
		}
	}else{
		QMessageBox mb("IRK","Couldn't open IrDA Serial Port.\nEither something else is using the Infrared, or you're using an OpenZaurus ROM with broken IrDA serial support.",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		mb.setIconPixmap(QPixmap((const char **)palm_xpm));
		mb.exec();
		m_on->setChecked(false);
	}
#endif
}

void CGIRK::patapataOpen()
{
	// Init the pocketop lookup table
	patapataInit();
	c700Init();
#if !(X86)
	// open the serial port
	fd=open(m_irdaDeviceNames[m_zaurusModel],O_RDONLY|O_NONBLOCK|O_NOCTTY);
	if(fd!=-1)
	{
		struct termios options;
		// Setup the serial port
		tcgetattr(fd, &options);
		
		options.c_iflag = IGNBRK;
		options.c_cflag = (CS8|CREAD|CLOCAL|CRTSCTS);
		options.c_oflag = (NL0|CR0|TAB0|BS0|VT0|FF0);
		options.c_lflag = 0;
		
		// 9600 baud
		cfsetispeed(&options, B9600);
		cfsetospeed(&options, B9600);
		
		// Set the new options for the port...
		tcsetattr(fd, TCSANOW, &options);
	
		// setup a socket notifier to tell us data is available
		sn = new QSocketNotifier( fd, QSocketNotifier::Read);
		if(sn)
		{
			connect( sn, SIGNAL(activated(int)), SLOT(dataAvailablePatapata(int)));
			m_keyboardOpen=true;
		}else{
			::close(fd);
			fd=-1;
		}
	}else{
		QMessageBox mb("IRK","Couldn't open IrDA Serial Port.\nEither something else is using the Infrared, or you're using an OpenZaurus ROM with broken IrDA serial support.",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		mb.setIconPixmap(QPixmap((const char **)targus_xpm));
		mb.exec();
		m_on->setChecked(false);
	}
#endif
}

void CGIRK::US102Open()
{
	// Init the pocketop lookup table
	US102Init();
	c700Init();
#if !(X86)
	// open the IRDA serial port
	fd=open(m_irdaDeviceNames[m_zaurusModel],O_RDONLY|O_NONBLOCK|O_NOCTTY);
	if(fd!=-1)
	{
		struct termios options;
		// Setup the Bluetooth serial port
		tcgetattr(fd, &options);
		
		options.c_iflag = IGNBRK;
		options.c_cflag = (CS8|CREAD|CLOCAL|CRTSCTS);
		options.c_oflag = (NL0|CR0|TAB0|BS0|VT0|FF0);
		options.c_lflag = 0;
		
		// 9600 baud
		cfsetispeed(&options, B9600);
		cfsetospeed(&options, B9600);
		
		// Set the new options for the port...
		tcsetattr(fd, TCSANOW, &options);
	
		// setup a socket notifier to tell us data is available
		sn = new QSocketNotifier( fd, QSocketNotifier::Read);
		if(sn)
		{
			connect( sn, SIGNAL(activated(int)), SLOT(dataAvailableUS102(int)));
			m_keyboardOpen=true;
		}else{
			::close(fd);
			fd=-1;
		}
	}else{
		QMessageBox mb("IRK","Couldn't open IrDA Serial Port.\nEither something else is using the Infrared, or you're using an OpenZaurus ROM with broken IrDA serial support.",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		mb.setIconPixmap(QPixmap((const char **)palm_xpm));
		mb.exec();
		m_on->setChecked(false);
	}
#endif
}

void CGIRK::JP106Open()
{
	// Init the pocketop lookup table
	JP106Init();
	c700Init();
#if !(X86)
	// open the IRDA serial port
	fd=open(m_irdaDeviceNames[m_zaurusModel],O_RDONLY|O_NONBLOCK|O_NOCTTY);
	if(fd!=-1)
	{
		struct termios options;
		// Setup the Bluetooth serial port
		tcgetattr(fd, &options);
		
		options.c_iflag = IGNBRK;
		options.c_cflag = (CS8|CREAD|CLOCAL|CRTSCTS);
		options.c_oflag = (NL0|CR0|TAB0|BS0|VT0|FF0);
		options.c_lflag = 0;
		
		// 9600 baud
		cfsetispeed(&options, B9600);
		cfsetospeed(&options, B9600);
		
		// Set the new options for the port...
		tcsetattr(fd, TCSANOW, &options);
	
		// setup a socket notifier to tell us data is available
		sn = new QSocketNotifier( fd, QSocketNotifier::Read);
		if(sn)
		{
			connect( sn, SIGNAL(activated(int)), SLOT(dataAvailableJP106(int)));
			m_keyboardOpen=true;
		}else{
			::close(fd);
			fd=-1;
		}
	}else{
		QMessageBox mb("IRK","Couldn't open IrDA Serial Port.\nEither something else is using the Infrared, or you're using an OpenZaurus ROM with broken IrDA serial support.",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		mb.setIconPixmap(QPixmap((const char **)palm_xpm));
		mb.exec();
		m_on->setChecked(false);
	}
#endif
}

void CGIRK::palmClose()
{
#if !(X86)
	if(fd!=-1)
	{
		// close the device
		::close(fd);
		fd=-1;
	}
#endif
}
void CGIRK::targusClose()
{
#if !(X86)
	if(fd!=-1)
	{
		// close the device
		::close(fd);
		fd=-1;
	}
#endif
}
void CGIRK::pocketopClose()
{
#if !(X86)
	if(fd!=-1)
	{
		// close the device
		::close(fd);
		fd=-1;
	}
#endif
}

void CGIRK::TKB780UClose()
{
#if !(X86)
	if(fd!=-1)
	{
		// close the device
		::close(fd);
		fd=-1;
	}
#endif
}

void CGIRK::F8U1500Close()
{
#if !(X86)
	if(fd!=-1)
	{
		// close the device
		::close(fd);
		fd=-1;
	}
#endif
}

void CGIRK::F8U1500TClose()
{
#if !(X86)
	if(fd!=-1)
	{
		// close the device
		::close(fd);
		fd=-1;
	}
#endif
}

void CGIRK::PalmOne3169WWClose()
{
#if !(X86)
	if(fd!=-1)
	{
		// close the device
		::close(fd);
		fd=-1;
	}
#endif
}

void CGIRK::HPfoldableBTClose()
{
#if !(X86)
	if(fd!=-1)
	{
		// close the device
		::close(fd);
		fd=-1;
	}
#endif
}

void CGIRK::patapataClose()
{
#if !(X86)
	if(fd!=-1)
	{
		// close the device
		::close(fd);
		fd=-1;
	}
#endif
}

void CGIRK::US102Close()
{
#if !(X86)
	if(fd!=-1)
	{
		// close the device
		::close(fd);
		fd=-1;
	}
#endif
}

void CGIRK::JP106Close()
{
#if !(X86)
	if(fd!=-1)
	{
		// close the device
		::close(fd);
		fd=-1;
	}
#endif
}

void CGIRK::keyboardOpen()
{
	if(m_keyboardOpen==false)
	{
		switch(m_currentKeyboard->mode())
		{
			case IRKMODE_LIRC:
				lircOpen();
				break;
			case IRKMODE_POCKETOP:
				pocketopOpen();
				break;
			case IRKMODE_IRDA:
				break;
			case IRKMODE_TARGUS:
				targusOpen();
				break;
			case IRKMODE_PALM:
				palmOpen();
				break;
			case IRKMODE_TKB780U:
				TKB780UOpen();
				break;
			case IRKMODE_F8U1500:
				F8U1500Open();
				break;
			case IRKMODE_F8U1500T:
				F8U1500TOpen();
				break;
			case IRKMODE_PalmOne3169WW:
				PalmOne3169WWOpen();
				break;
			case IRKMODE_HPfoldableBT:
				HPfoldableBTOpen();
				break;
			case IRKMODE_PATAPATA:
				patapataOpen();
				break;
			case IRKMODE_US102:
				US102Open();
				break;
			case IRKMODE_JP106:
				JP106Open();
				break;
		}
	}
}

void CGIRK::keyboardClose()
{
	if(m_keyboardOpen==true)
	{
		if(sn)
		{
			// delete the socket notifier
			delete sn;
			sn=NULL;
		}
		switch(m_currentKeyboard->mode())
		{
			case IRKMODE_LIRC:
				lircClose();
				break;
			case IRKMODE_TARGUS:
				targusClose();
				break;
			case IRKMODE_PALM:
				palmClose();
				break;
			case IRKMODE_TKB780U:
				TKB780UClose();
				break;
			case IRKMODE_F8U1500:
				F8U1500Close();
				break;
			case IRKMODE_F8U1500T:
				F8U1500TClose();
				break;
			case IRKMODE_PalmOne3169WW:
				PalmOne3169WWClose();
				break;
			case IRKMODE_HPfoldableBT:
				HPfoldableBTClose();
				break;
			case IRKMODE_PATAPATA:
				patapataClose();
				break;
			case IRKMODE_POCKETOP:
				pocketopClose();
				break;
			case IRKMODE_IRDA:
				break;
			case IRKMODE_US102:
				US102Close();
				break;
			case IRKMODE_JP106:
				JP106Close();
				break;
		}
		m_keyboardOpen=false;
	}
}

void CGIRK::readConfig()
{
	// clear the keyboard list ready for the new one to be loaded in
	m_keyboardList.clear();
#if !POCKETOP_BUILD
	pSettingsForm->mKeyboard->clear();
#endif
	Config config("IRK");
	config.setGroup("Config");
	
	bool defOn=config.readBoolEntry ( "DefaultOn", false);
	
	pSettingsForm->mDefaultOn->setChecked(defOn);
	pSettingsForm->mDefaultOff->setChecked(!defOn);
	
	// read number of keyboards supports
	int nKeyboards=config.readNumEntry("NumKeyboards",0);

	// read key repeat time
	m_keyRepeatEnable=config.readBoolEntry("KeyRepeatEnable",true);
	pSettingsForm->mRepeatEnable->setChecked(m_keyRepeatEnable);
	m_keyRepeatPeriod=config.readNumEntry("KeyRepeatPeriod",500);
	pSettingsForm->mRepeatPeriod->setValue(m_keyRepeatPeriod);
	m_keyRepeatDelay=config.readNumEntry("KeyRepeatDelay",500);
	pSettingsForm->mRepeatDelay->setValue(m_keyRepeatDelay);

	// read swap ctrl-caps
	m_swapCtrlCaps=config.readBoolEntry("SwapCtrlCaps",true);
	pSettingsForm->mSwapCtrlCaps->setChecked(m_swapCtrlCaps);

	// Read zaurus model (as IrDA serial port is a different device on SL5500 vs Sl5600)
	m_zaurusModel=(ZAURUS_MODEL)config.readNumEntry("ZaurusModel",0);
	pSettingsForm->mZaurusModel->setCurrentItem(m_zaurusModel);
	
	// read names of keyboards
	QString keyboardName,kEntry;
	int k;
	for(k=0; k<nKeyboards; k++)
	{
		kEntry.sprintf("Keyboard[%d]",k);
		keyboardName=config.readEntry(kEntry);
		m_keyboardList.append(new CIRKeyboard(keyboardName));
	}
	
	int defaultKeyboardID=config.readNumEntry("DefaultKeyboard",0);
	m_currentKeyboard=m_keyboardList.at(defaultKeyboardID);
	
	// read config for each keyboard
	for(k=0; k<nKeyboards; k++)
	{
		config.setGroup(m_keyboardList.at(k)->name());
		QString mode=config.readEntry("Mode","LIRC");
		if(mode=="LIRC")
		{
			m_keyboardList.at(k)->setMode(IRKMODE_LIRC);
			m_keyboardList.at(k)->setLIRCConf(config.readEntry("LIRC_Config",""));
		}else if(mode=="POCKETOP"){
			m_keyboardList.at(k)->setMode(IRKMODE_POCKETOP);
		}else if(mode=="TARGUS"){
			m_keyboardList.at(k)->setMode(IRKMODE_TARGUS);
		}else if(mode=="PALM"){
			m_keyboardList.at(k)->setMode(IRKMODE_PALM);
		}else if(mode=="TKB780U"){
			m_keyboardList.at(k)->setMode(IRKMODE_TKB780U);
		}else if(mode=="F8U1500"){
			m_keyboardList.at(k)->setMode(IRKMODE_F8U1500);
		}else if(mode=="F8U1500T"){
			m_keyboardList.at(k)->setMode(IRKMODE_F8U1500T);
		}else if(mode=="PalmOne3169WW"){
			m_keyboardList.at(k)->setMode(IRKMODE_PalmOne3169WW);
		}else if(mode=="HPfoldableBT"){
			m_keyboardList.at(k)->setMode(IRKMODE_HPfoldableBT);
		}else if(mode=="PATAPATA"){
			m_keyboardList.at(k)->setMode(IRKMODE_PATAPATA);
		}else if(mode=="US102"){
			m_keyboardList.at(k)->setMode(IRKMODE_US102);
		}else if(mode=="JP106"){
			m_keyboardList.at(k)->setMode(IRKMODE_JP106);
		}else{
			m_keyboardList.at(k)->setMode(IRKMODE_IRDA);
		}
#if !POCKETOP_BUILD
		if (m_keyboardList.at(k)->mode() == IRKMODE_POCKETOP)
			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)pocketop_xpm), m_keyboardList.at(k)->name());
		else if (m_keyboardList.at(k)->mode() == IRKMODE_TARGUS)
			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)targus_xpm), m_keyboardList.at(k)->name());
		else if (m_keyboardList.at(k)->mode() == IRKMODE_PALM)
			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)palm_xpm), m_keyboardList.at(k)->name());
		else if (m_keyboardList.at(k)->mode() == IRKMODE_TKB780U)
			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)palm_xpm), m_keyboardList.at(k)->name());
		else if (m_keyboardList.at(k)->mode() == IRKMODE_F8U1500)
			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)palm_xpm), m_keyboardList.at(k)->name());
		else if (m_keyboardList.at(k)->mode() == IRKMODE_F8U1500T)
			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)palm_xpm), m_keyboardList.at(k)->name());
		else if (m_keyboardList.at(k)->mode() == IRKMODE_PalmOne3169WW)
			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)palm_xpm), m_keyboardList.at(k)->name());
		else if (m_keyboardList.at(k)->mode() == IRKMODE_HPfoldableBT)
			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)palm_xpm), m_keyboardList.at(k)->name());
		else if (m_keyboardList.at(k)->mode() == IRKMODE_PATAPATA)
			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)targus_xpm), m_keyboardList.at(k)->name());
		else if (m_keyboardList.at(k)->mode() == IRKMODE_US102)
//			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)ps2_xpm), m_keyboardList.at(k)->name());
			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)palm_xpm), m_keyboardList.at(k)->name());
		else if (m_keyboardList.at(k)->mode() == IRKMODE_JP106)
//			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)ps2_xpm), m_keyboardList.at(k)->name());
			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)palm_xpm), m_keyboardList.at(k)->name());
		else	
			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)Infrared_xpm), m_keyboardList.at(k)->name());
#endif
		int numShifted=config.readNumEntry("NumShifted",0);
		m_keyboardList.at(k)->setNumShifted(numShifted);
		int numPunct=config.readNumEntry("NumPunct",0);
		m_keyboardList.at(k)->setNumPunct(numPunct);
		int numCmd=config.readNumEntry("NumCmd",0);
		m_keyboardList.at(k)->setNumCmd(numCmd);
		int numFnl=config.readNumEntry("NumFnl",0);
		m_keyboardList.at(k)->setNumFnl(numFnl);
		int numFnr=config.readNumEntry("NumFnr",0);
		m_keyboardList.at(k)->setNumFnr(numFnr);
		int numNum=config.readNumEntry("NumNum",0);
		m_keyboardList.at(k)->setNumNum(numNum);
		int numFast=config.readNumEntry("NumFast",0);
		m_keyboardList.at(k)->setNumFast(numFast);
		int n,sep;
		QString code,sEntry;
		QChar key;
		for(n=0; n<numShifted; n++)
		{
			sEntry.sprintf("S[%d]",n);
			code=config.readEntry(sEntry,"0:#0");
			sep=code.find(':');
			key=(QChar)code.left(sep).toInt();
			m_keyboardList.at(k)->setShiftMapEntry(key,code.mid(sep+1));
		}
		for(n=0; n<numPunct; n++)
		{
			sEntry.sprintf("P[%d]",n);
			code=config.readEntry(sEntry,"0:#0");
			sep=code.find(':');
			key=(QChar)code.left(sep).toInt();
			m_keyboardList.at(k)->setPunctMapEntry(key,code.mid(sep+1));
		}
		for(n=0; n<numCmd; n++)
		{
			sEntry.sprintf("C[%d]",n);
			code=config.readEntry(sEntry,"0:#0");
			sep=code.find(':');
			key=(QChar)code.left(sep).toInt();
			m_keyboardList.at(k)->setCmdMapEntry(key,code.mid(sep+1));
		}
		for(n=0; n<numFnl; n++)
		{
			sEntry.sprintf("L[%d]",n);
			code=config.readEntry(sEntry,"0:#0");
			sep=code.find(':');
			key=(QChar)code.left(sep).toInt();
			m_keyboardList.at(k)->setFnlMapEntry(key,code.mid(sep+1));
		}
		for(n=0; n<numFnr; n++)
		{
			sEntry.sprintf("R[%d]",n);
			code=config.readEntry(sEntry,"0:#0");
			sep=code.find(':');
			key=(QChar)code.left(sep).toInt();
			m_keyboardList.at(k)->setFnrMapEntry(key,code.mid(sep+1));
		}
		for(n=0; n<numNum; n++)
		{
			sEntry.sprintf("N[%d]",n);
			code=config.readEntry(sEntry,"0:#0");
			sep=code.find(':');
			key=(QChar)code.left(sep).toInt();
			m_keyboardList.at(k)->setNumMapEntry(key,code.mid(sep+1));
		}
		for(n=0; n<numFast; n++)
		{
			sEntry.sprintf("F[%d]",n);
			code=config.readEntry(sEntry,"0:#0");
			sep=code.find(':');
			key=(QChar)code.left(sep).toInt();
			m_keyboardList.at(k)->setFastMapEntry(key,code.mid(sep+1));
		}
	}
	
	pSettingsForm->changedKeyboard(defaultKeyboardID);
#if !POCKETOP_BUILD
	pSettingsForm->mKeyboard->setCurrentItem(m_keyboardList.find(m_currentKeyboard));
#endif
}

void CGIRK::writeConfig()
{
	Config config("IRK");
	config.setGroup("Config");
	config.writeEntry ( "DefaultOn", pSettingsForm->mDefaultOn->isChecked());
	int nKeyboards=m_keyboardList.count();
	config.writeEntry("NumKeyboards",nKeyboards);
	config.writeEntry("KeyRepeatEnable",m_keyRepeatEnable);
	config.writeEntry("KeyRepeatPeriod",m_keyRepeatPeriod);
	config.writeEntry("KeyRepeatDelay",m_keyRepeatDelay);
	config.writeEntry("SwapCtrlCaps", m_swapCtrlCaps);
	config.writeEntry("ZaurusModel",(int)m_zaurusModel);
	int k;
	for(k=0; k<nKeyboards; k++)
	{
		QString kEntry;
		kEntry.sprintf("Keyboard[%d]",k);
		config.writeEntry(kEntry,m_keyboardList.at(k)->name());
	}
	config.writeEntry("DefaultKeyboard",m_keyboardList.find(m_currentKeyboard));
	for(k=0; k<nKeyboards; k++)
	{
		config.setGroup(m_keyboardList.at(k)->name());
		if(m_keyboardList.at(k)->mode()==IRKMODE_LIRC)
		{
			config.writeEntry("Mode","LIRC");
			config.writeEntry("LIRC_Config",m_keyboardList.at(k)->lircConf());
		}else if(m_keyboardList.at(k)->mode()==IRKMODE_IRDA){
			config.writeEntry("Mode","IRDA");
		}else if(m_keyboardList.at(k)->mode()==IRKMODE_POCKETOP){
			config.writeEntry("Mode","POCKETOP");
		}else if(m_keyboardList.at(k)->mode()==IRKMODE_TARGUS){
			config.writeEntry("Mode","TARGUS");
		}else if(m_keyboardList.at(k)->mode()==IRKMODE_PALM){
			config.writeEntry("Mode","PALM");
		}else if(m_keyboardList.at(k)->mode()==IRKMODE_TKB780U){
			config.writeEntry("Mode","TKB780U");
		}else if(m_keyboardList.at(k)->mode()==IRKMODE_F8U1500){
			config.writeEntry("Mode","F8U1500");
		}else if(m_keyboardList.at(k)->mode()==IRKMODE_F8U1500T){
			config.writeEntry("Mode","F8U1500T");
		}else if(m_keyboardList.at(k)->mode()==IRKMODE_PalmOne3169WW){
			config.writeEntry("Mode","PalmOne3169WW");
		}else if(m_keyboardList.at(k)->mode()==IRKMODE_HPfoldableBT){
			config.writeEntry("Mode","HPfoldableBT");
		}else if(m_keyboardList.at(k)->mode()==IRKMODE_PATAPATA){
			config.writeEntry("Mode","PATAPATA");
		}else if(m_keyboardList.at(k)->mode()==IRKMODE_US102){
			config.writeEntry("Mode","US102");
		}else if(m_keyboardList.at(k)->mode()==IRKMODE_JP106){
			config.writeEntry("Mode","JP106");
		}
		config.writeEntry("NumShifted",m_keyboardList.at(k)->numShifted());
		config.writeEntry("NumCmd",m_keyboardList.at(k)->numCmd());
		config.writeEntry("NumFnl",m_keyboardList.at(k)->numFnl());
		config.writeEntry("NumFnr",m_keyboardList.at(k)->numFnr());
		config.writeEntry("NumPunct",m_keyboardList.at(k)->numPunct());
		config.writeEntry("NumNum",m_keyboardList.at(k)->numNum());
		config.writeEntry("NumFast",m_keyboardList.at(k)->numFast());
		int n,count;
		count=0;
		for(n=0; n<256; n++)
		{
			if(m_keyboardList.at(k)->shiftMapEntry(n).length())
			{
				QString sEntry;
				QString sCode;
				sEntry.sprintf("S[%d]",count);
				sCode.sprintf("%d:%s",n,(const char*)(m_keyboardList.at(k)->shiftMapEntry(n)).utf8());
				config.writeEntry(sEntry,sCode);
				count++;
			}
		}
		count=0;
		for(n=0; n<256; n++)
		{
			if(m_keyboardList.at(k)->cmdMapEntry(n).length())
			{
				QString sEntry;
				QString sCode;
				sEntry.sprintf("C[%d]",count);
				sCode.sprintf("%d:%s",n,(const char*)(m_keyboardList.at(k)->cmdMapEntry(n)).utf8());
				config.writeEntry(sEntry,sCode);
				count++;
			}
		}
		count=0;
		for(n=0; n<256; n++)
		{
			if(m_keyboardList.at(k)->fnlMapEntry(n).length())
			{
				QString sEntry;
				QString sCode;
				sEntry.sprintf("L[%d]",count);
				sCode.sprintf("%d:%s",n,(const char*)(m_keyboardList.at(k)->fnlMapEntry(n)).utf8());
				config.writeEntry(sEntry,sCode);
				count++;
			}
		}
		count=0;
		for(n=0; n<256; n++)
		{
			if(m_keyboardList.at(k)->fnrMapEntry(n).length())
			{
				QString sEntry;
				QString sCode;
				sEntry.sprintf("R[%d]",count);
				sCode.sprintf("%d:%s",n,(const char*)(m_keyboardList.at(k)->fnrMapEntry(n)).utf8());
				config.writeEntry(sEntry,sCode);
				count++;
			}
		}
		count=0;
		for(n=0; n<256; n++)
		{
			if(m_keyboardList.at(k)->punctMapEntry(n).length())
			{
				QString sEntry;
				QString sCode;
				sEntry.sprintf("P[%d]",count);
				sCode.sprintf("%d:%s",n,(const char*)(m_keyboardList.at(k)->punctMapEntry(n)).utf8());
				config.writeEntry(sEntry,sCode);
				count++;
			}
		}
		count=0;
		for(n=0; n<256; n++)
		{
			if(m_keyboardList.at(k)->numMapEntry(n).length())
			{
				QString sEntry;
				QString sCode;
				sEntry.sprintf("N[%d]",count);
				sCode.sprintf("%d:%s",n,(const char*)(m_keyboardList.at(k)->numMapEntry(n)).utf8());
				config.writeEntry(sEntry,sCode);
				count++;
			}
		}
		count=0;
		for(n=0; n<256; n++)
		{
			if(m_keyboardList.at(k)->fastMapEntry(n).length())
			{
				QString sEntry;
				QString sCode;
				sEntry.sprintf("F[%d]",count);
				sCode.sprintf("%d:%s",n,(const char*)(m_keyboardList.at(k)->fastMapEntry(n)).utf8());
				config.writeEntry(sEntry,sCode);
				count++;
			}
		}
	}
}

void CGIRK::settings()
{
	pSettingsForm->showMaximized();
	pSettingsForm->changedKeyboard(pSettingsForm->mKeyboard->currentItem());
	
	if(pSettingsForm->exec()==QDialog::Accepted)
	{
		bool isOpen=m_keyboardOpen;
		keyboardClose();
#if !POCKETOP_BUILD
		int newKeyboardId=pSettingsForm->mKeyboard->currentItem();
		m_currentKeyboard=m_keyboardList.at(newKeyboardId);
#endif
		m_keyRepeatPeriod=pSettingsForm->mRepeatPeriod->value();
		m_keyRepeatDelay=pSettingsForm->mRepeatDelay->value();
		m_keyRepeatEnable=pSettingsForm->mRepeatEnable->isChecked();
		m_swapCtrlCaps=pSettingsForm->mSwapCtrlCaps->isChecked();
		m_zaurusModel=(ZAURUS_MODEL)pSettingsForm->mZaurusModel->currentItem();
		m_currentKeyboard->resetMaps();
		
		QListViewItem *pI;
		for(pI=pSettingsForm->mShiftKeyListView->firstChild(); pI; pI=pI->nextSibling())
		{
			QChar ch=specialForName(pI->text(0));
			if(ch)
			{
				m_currentKeyboard->setShiftMapEntry(ch,pI->text(1));
				m_currentKeyboard->setNumShifted(m_currentKeyboard->numShifted()+1);
			}
		}
		for(pI=pSettingsForm->mNumKeyListView->firstChild(); pI; pI=pI->nextSibling())
		{
			QChar ch=specialForName(pI->text(0));
			if(ch)
			{
				m_currentKeyboard->setNumMapEntry(ch,pI->text(1));
				m_currentKeyboard->setNumNum(m_currentKeyboard->numNum()+1);
			}
		}
		for(pI=pSettingsForm->mCmdKeyListView->firstChild(); pI; pI=pI->nextSibling())
		{
			QChar ch=specialForName(pI->text(0));
			if(ch)
			{
				m_currentKeyboard->setCmdMapEntry(ch,pI->text(1));
				m_currentKeyboard->setNumCmd(m_currentKeyboard->numCmd()+1);
			}
		}
		for(pI=pSettingsForm->mFnlKeyListView->firstChild(); pI; pI=pI->nextSibling())
		{
			QChar ch=specialForName(pI->text(0));
			if(ch)
			{
				m_currentKeyboard->setFnlMapEntry(ch,pI->text(1));
				m_currentKeyboard->setNumFnl(m_currentKeyboard->numFnl()+1);
			}
		}
		for(pI=pSettingsForm->mFnrKeyListView->firstChild(); pI; pI=pI->nextSibling())
		{
			QChar ch=specialForName(pI->text(0));
			if(ch)
			{
				m_currentKeyboard->setFnrMapEntry(ch,pI->text(1));
				m_currentKeyboard->setNumFnr(m_currentKeyboard->numFnr()+1);
			}
		}
		for(pI=pSettingsForm->mPunctKeyListView->firstChild(); pI; pI=pI->nextSibling())
		{
			QChar ch=specialForName(pI->text(0));
			if(ch)
			{
				m_currentKeyboard->setPunctMapEntry(ch,pI->text(1));
				m_currentKeyboard->setNumPunct(m_currentKeyboard->numPunct()+1);
			}
		}
		for(pI=pSettingsForm->mFastKeyListView->firstChild(); pI; pI=pI->nextSibling())
		{
			QChar ch=specialForName(pI->text(0));
			if(ch)
			{
				m_currentKeyboard->setFastMapEntry(ch,pI->text(1));
				m_currentKeyboard->setNumFast(m_currentKeyboard->numFast()+1);
			}
		}
		
		if(isOpen)
			keyboardOpen();
		writeConfig();
	}
}


void CGIRK::cancelShift()
{
	m_shiftTimer->stop();
	m_isShiftedl=m_isShiftedr=false;
	emit key(0,Key_Shift,0,false,0);
}

void CGIRK::setRepeat(int ms,int delay)
{
	m_keyRepeatPeriod=ms;
	if(delay>0)
		m_keyRepeatDelay=delay;
}


void CGIRK::caps(bool state)
{
	m_capsLock=state;
}

void CGIRK::num(bool state)
{
	m_numLock=state;
}

void CGIRK::on(bool state)
{
	if(state)
	{
		keyboardOpen();
	}
}

void CGIRK::off(bool state )
{
	if(state)
	{
		keyboardClose();
		m_lastDown1=m_lastDown2=0;
		m_repeatTimer->stop();
		m_caps->setChecked(false);
		m_num->setChecked(false);
		m_capsLock=m_numLock=m_isShiftedl=m_isShiftedr=m_isNum=m_isPunct=m_isCmd=m_isCtrl=m_isAlt=false;
		m_isFnl=m_isFnr=m_isVirtualfn=false;
	}
}


QSize CGIRK::sizeHint() const
{
    return QSize(320,height());
}

QString CGIRK::getShifted(char c)
{
	return m_currentKeyboard->shiftMapEntry(c);
}

QString CGIRK::getNum(char c)
{
	return m_currentKeyboard->numMapEntry(c);
}

QString CGIRK::getPunct(char c)
{
	return m_currentKeyboard->punctMapEntry(c);
}

QString CGIRK::getCmd(char c)
{
	return m_currentKeyboard->cmdMapEntry(c);
}

QString CGIRK::getFnl(char c)
{
	return m_currentKeyboard->fnlMapEntry(c);
}

QString CGIRK::getFnr(char c)
{
	return m_currentKeyboard->fnrMapEntry(c);
}

QString CGIRK::getFast(char c)
{
	return m_currentKeyboard->fastMapEntry(c);
}


bool CGIRK::isNum()
{
	return m_isNum||m_numLock;
}

void CGIRK::handleKeyPress(unsigned short c,bool down)
{

	QString macro;
	static short m_lastMod_tmp;
	static bool m_isShiftedr_tmp;
	static bool m_isShiftedl_tmp;
	m_repeatTimer->stop();

	if ( m_swapCtrlCaps )
	{
		if ( c == CGIRK_KEY_CTRL )
			c = CGIRK_KEY_CAPS;
		else if ( c == CGIRK_KEY_CAPS )
			c = CGIRK_KEY_CTRL;
	}

	if(c<=30)
	{
		switch(c)
		{
			case CGIRK_KEY_SHIFTL:
				if(m_isCmd)
				{
					m_isCmd=!down;
					m_isVirtualfn=down;
					m_isShiftedl_tmp=down;
					m_lastMod_tmp=m_lastMod;
					m_lastMod=0;
					emit key(0xFFFF,Key_F22,0,down,0);
				}else if(m_isVirtualfn&&m_isShiftedl_tmp){
					emit key(0xFFFF,Key_F22,0,down,0);
					m_isVirtualfn=down;
					m_isCmd=!down;
					m_isShiftedl_tmp=down;	
					m_lastMod=m_lastMod_tmp;
				}else if(m_isVirtualfn&&m_isShiftedr_tmp){
					m_isShiftedl_tmp=down;
				}else{
					m_isShiftedl=down;
					m_isShiftedl_tmp=down;
					emit key(0xFFFF,Key_Shift,m_lastMod,down,0);
				}
				if(!(m_lastMod&(Qt::ControlButton|Qt::AltButton)))
					c=0;
				break;

			case CGIRK_KEY_SHIFTR:
				if(m_isCmd)
				{
					m_isCmd=!down;
					m_isVirtualfn=down;
					m_isShiftedr_tmp=down;
					m_lastMod_tmp=m_lastMod;
					m_lastMod=0;
					emit key(0xFFFF,Key_F22,0,down,0);
				}else if(m_isVirtualfn&&m_isShiftedr_tmp){
					emit key(0xFFFF,Key_F22,0,down,0);
					m_isVirtualfn=down;
					m_isCmd=!down;
					m_isShiftedr_tmp=down;
					m_lastMod=m_lastMod_tmp;
				}else if(m_isVirtualfn&&m_isShiftedl_tmp){
					m_isShiftedr_tmp=down;
				}else{
					m_isShiftedr=down;
					m_isShiftedr_tmp=down;
					emit key(0xFFFF,Key_Shift,m_lastMod,down,0);
				}
				if(!(m_lastMod&(Qt::ControlButton|Qt::AltButton)))
					c=0;
				break;

			case CGIRK_KEY_CTRL:
				m_isCtrl=down;
				emit key(0xFFFF,Key_Control,m_lastMod,down,0);
				c=0;
				break;

			case CGIRK_KEY_PUNCT:
				m_isPunct=down;
				c=0;
				break;

			case CGIRK_KEY_CMD:
				if(m_isShiftedl|m_isShiftedr){
					emit key(0xFFFF,Key_Shift,m_lastMod,!down,0);
					m_lastMod_tmp=m_lastMod;
					m_lastMod=0;
					m_isVirtualfn=down;
					m_isShiftedr_tmp=m_isShiftedr;
					m_isShiftedl_tmp=m_isShiftedl;
					m_isShiftedr=false;
					m_isShiftedl=false;
					emit key(0xFFFF,Key_F22,m_lastMod,down,0);
				}else if(m_isVirtualfn){
					emit key(0xFFFF,Key_F22,m_lastMod,down,0);
					m_lastMod=m_lastMod_tmp;
					m_isVirtualfn=down;
					m_isShiftedr=m_isShiftedr_tmp;
					m_isShiftedl=m_isShiftedl_tmp;
					emit key(0xFFFF,Key_Shift,m_lastMod,!down,0);
				}else{
					m_isCmd=down;
				}
				c=0;
				break;

			case CGIRK_KEY_FNL:
				m_isFnl=down;
				c=0;
				break;

			case CGIRK_KEY_FNR:
				m_isFnr=down;
				c=0;
				break;

			case CGIRK_KEY_NUM:
				m_isNum=down;
				emit key(0xFFFF,Key_NumLock,m_lastMod,down,0);
				if(down)
					m_lastDown2=Key_NumLock;
				else if(m_lastDown2==Key_NumLock)
				{
					m_numLock=!m_numLock;
					m_num->setChecked(m_numLock);
					m_lastDown2=0xFFFF;
				}
				c=0;
				break;

			case CGIRK_KEY_ALT:
				m_isAlt=down;
				emit key(0xFFFF,Key_Alt,m_lastMod,down,0);
				c=0;
				break;

			case CGIRK_KEY_HOME:
					emit key(0xFFFF,Key_Home,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_Home);
					}
					break;
			case CGIRK_KEY_LEFT:
				if(m_isCmd)
				{
					emit key(0xFFFF,Key_Home,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_Home);
					}
				}else if(isNum()){
					emit key('.','.',m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat('.','.');
					}
				}else if(m_isFnl){
					emit key(0xFFFF,Key_Home,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_Home);
					}
				}else if(m_isFnr){
					emit key(0xFFFF,Key_Home,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_Home);
					}
				}else{
					emit key(0xFFFF,Key_Left,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_Left);
					}
				}
				c=0;
				break;
			case CGIRK_KEY_RIGHT:
				if(m_isCmd)
				{
					emit key(0xFFFF,Key_End,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_End);
					}
				}else if(isNum()){
					emit key('=','=',m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat('=','=');
					}
				}else if(m_isFnl){
					emit key(0xFFFF,Key_End,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_End);
					}
				}else if(m_isFnr){
					emit key(0xFFFF,Key_End,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_End);
					}
				}else{
					emit key(0xFFFF,Key_Right,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_Right);
					}
				}
				c=0;
				break;
			case CGIRK_KEY_UP:
				if(m_isCmd)
				{
					emit key(0xFFFF,Key_PageUp,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_PageUp);
					}
				}else if(isNum()){
					emit key('-','-',m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat('-','-');
					}
				}else if(m_isShiftedr){
					emit key('?','?',m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat('?','?');
					}
				}else if(m_isFnl){
					emit key(0xFFFF,Key_PageUp,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_PageUp);
					}
				}else if(m_isFnr){
					emit key(0xFFFF,Key_PageUp,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_PageUp);
					}
				}else{
					emit key(0xFFFF,Key_Up,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_Up);
					}
				}
				c=0;
				break;
			case CGIRK_KEY_DOWN:
				if(m_isCmd)
				{
					emit key(0xFFFF,Key_PageDown,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_PageDown);
					}
				}else if(isNum()){
					emit key('+','+',m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat('+','+');
					}
				}else if(m_isFnl){
					emit key(0xFFFF,Key_PageDown,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_PageDown);
					}
				}else if(m_isFnr){
					emit key(0xFFFF,Key_PageDown,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_PageDown);
					}
				}else{
					emit key(0xFFFF,Key_Down,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_Down);
					}
				}
				c=0;
				break;
				
								
			default:
				{
					if(!(m_isNum|m_isAlt|m_isCmd))
					{
						// stanard key behaviours (no modifiers)
						switch(c)
						{
							case CGIRK_KEY_ESC:
								emit key(0xFFFF,Key_Escape,m_lastMod,down,0);
								if(down)
								{
									startKeyRepeat(0xFFFF,Key_Escape);
								}
								c=0;
								break;
							
					 		case CGIRK_KEY_CAPS:
								// toggle the caps lock bit
								if(m_isFnl)
								{
									if(down)
									{
										m_numLock=!m_numLock;
										m_num->setChecked(m_numLock);
									}
									emit key(0xFFFF,Key_NumLock,m_lastMod,down,0);
								}else{
									if(down)
									{
										m_capsLock=!m_capsLock;
										m_caps->setChecked(m_capsLock);
									}
									emit key(0xFFFF,Key_CapsLock,m_lastMod,down,0);
								}
								c=0;
								break;

							case CGIRK_KEY_FUNCTION:
								if(isNum())
									emit key('0','0',m_lastMod,down,0);
								else
									emit key(0xFFFF,Key_F12,m_lastMod,down,0);
								c=0;
								break;

							case CGIRK_KEY_DEL:
								if(m_isFnl)
								{
									emit key(0xFFFF,Key_F35,m_lastMod,down,0);
								}else{
									emit key(0x7f,Key_Delete,m_lastMod,down,0);
									if(down)
									{
										startKeyRepeat(0x7f,Key_Delete);
									}
								}
								c=0;
								break;
							case CGIRK_KEY_TAB:
								if(m_isCmd|m_isFnl)
								{
									emit key(0xFFFF,Key_Escape,m_lastMod,down,0);
								}else{
									emit key('\t',Key_Tab,m_lastMod,down,0);
									if(down)
									{
										startKeyRepeat('\t',Key_Tab);
									}
								}
								c=0;
								break;
							case CGIRK_KEY_BACKSPACE:
								if(m_isVirtualfn)
								{
									emit key(0x7f,Key_Delete,m_lastMod,down,0);
									if(down)
										startKeyRepeat(0x7f,Key_Delete);
								}else if(m_isCmd){
									// CMD+Backspace does suspend, so send F34 as that's the code sent when you hold down the cancel button
									//emit key(0xFFFF,Key_F34,m_lastMod,down,0);
								}else{
									emit key('\b',Key_Backspace,m_lastMod,down,0);
									if(down)
										startKeyRepeat('\b',Key_Backspace);
								}
								c=0;
								break;
							case CGIRK_KEY_CONFIRM:
								emit key(0xFFFF,Key_F33,m_lastMod,down,0);
								if(down)
								{
									startKeyRepeat(0xFFFF,Key_F33);
								}
								c=0;
								break;
							case CGIRK_KEY_RETURN:
								if(m_isCmd|m_isFnl)
								{
									// CMD+Enter==OK, send F33 as Zaurus uses that as the OK button
									emit key(0xFFFF,Key_F33,m_lastMod,down,0);
								}else{
									emit key('\r',Key_Return,m_lastMod,down,0);
									if(down)
									{
										startKeyRepeat('\r',Key_Return);
									}
								}
								c=0;
								break;
				/*			case CGIRK_KEY_DATE:
								if(m_isCmd)
									emit key(0xFFFF,Key_F11,m_lastMod,down,0);
								else
									emit key(0xFFFF,Key_F9,m_lastMod,down,0);
								break;
							case CGIRK_KEY_ADDRESS:
								if(m_isCmd)
									emit key(0xFFFF,Key_Menu,m_lastMod,down,0);
								else
									emit key(0xFFFF,Key_F10,m_lastMod,down,0);
								break;
							case CGIRK_KEY_TODO:        
								if(down)
								{
									if(m_isCmd)
										Global::execute("calculator");
									else
										Global::execute("todolist");
								}
								break;
							case CGIRK_KEY_MEMO:
								if(down)
									Global::execute("textedit");*/
								break;
							default:
								break;
						}
					}
				}
				break;
		}
		m_lastMod=modifiers();
	}
	
	if(c)
	{
		if(((c>='a')&&(c<='z'))&&(!(isNum()|m_isPunct|m_isCmd|m_isFnl|m_isFnr|m_isVirtualfn)))
		{
			if(m_capsLock)
				c&=~32;
			if(m_isShiftedl|m_isShiftedr)
				c^=32;
		}else{
			if((c==' ')&&(down)&&(m_isVirtualfn))
			{
				c=0;
				m_isRotated=!m_isRotated;

				if(m_isRotated)
				{
					QPEApplication::setDefaultRotation((m_rotation==180)?270:180);
					QMessageBox mb("IRK","Rotation: On",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
					mb.exec();
				}else{
					QPEApplication::setDefaultRotation(m_rotation);
					QMessageBox mb("IRK","Rotation: Off.",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
					mb.exec();
				}
			}else{
				if(isNum())
				{
					macro=getNum(c);
				}else if(m_isPunct){
					macro=getPunct(c);
				}else if(m_isShiftedl|m_isShiftedr){
					macro=getShifted(c);
				}else if(m_isCmd){
					macro=getCmd(c);
				}else if(m_isFnl){
					macro=getFnl(c);
				}else if(m_isFnr){
					macro=getFnr(c);
				}else{
					macro=getFast(c);
				}
			}
			
			int l=macro.length();

			if(l)
			{
				if((l>1)&&(macro[0]=='#'))
				{
					// Numeric macro
					if(macro[1]=='#')
					{
						// escaped numeric macro
						c='#';
					}else{
						// convert numeric  from number to character...
						macro=macro.mid(1);
						c=macro.toUInt();
					}
				}else if((l>1)&&(macro[0]==':')&&(down)){
					// QCOP macro
					if(macro[1]==':')
					{
						// escaped QCOP macro
						c=':';
					}else{
						// extract QCOP stuff from macro
						int i;
						macro=macro.mid(1);
						i=macro.find(":");
						QString channel=macro.left(i);
						macro=macro.mid(i+1);
						i=macro.find(":");
						QString service=macro.left(i);
						macro=macro.mid(i+1);
						QCopEnvelope e(channel.utf8(), service.utf8());
						QString arg;
						while(macro.length())
						{
							i=macro.find(":");
							if(i>=0)
							{
								arg=macro.left(i);
								macro=macro.mid(i+1);
								e << arg.utf8();
							}else{
								e << macro.utf8();
								break;
							}
						}
						c=0;
						m_lastDown1=m_lastDown2=0;
					}
				}else if((l>1)&&(macro[0]=='*')&&(down)){
					// Execute macro
					if(macro[1]=='*')
					{
						// escaped execute macro
						c='*';
					}else{
						// execute command given in macro...
						QString arg = "";
						int idx = macro.find(' ',1);

						if ( idx < 0 ) {
							macro = macro.mid(1);
						} else {
							arg = macro.mid(idx+1);
							macro = macro.mid(1,idx-1);
						} 
						Global::execute(macro,arg);

						c=0;
						m_lastDown1=m_lastDown2=0;
					}
				}else if((l>1)&&(macro[0]=='%')&&(down)){
					// Execute Qtopia Applications
					if(macro[1]=='%')
					{
						// escaped execute macro
						c='%';
					}else{
						// execute Qtopia Applications given in macro...
						QString arg = "";
						int idx = macro.find(' ',1);

						if ( idx < 0 ) {
							macro = macro.mid(1);
						} else {
							arg = macro.mid(idx+1);
							macro = macro.mid(1,idx-1);
						} 
						AppLnk alnk = AppLnk(QPEApplication::qpeDir()+"apps/"+macro+".desktop");
						alnk.execute( arg );

						c=0;
						m_lastDown1=m_lastDown2=0;
					}
				}else if((l>1)&&(macro[0]=='<')&&(down)){
					// Execute Paste by Clipboard
					if(macro[1]=='<')
					{
						// escaped this macro
						c='<';
					}else{
						// execute Paste by Clipboard
						QClipboard* cb = QApplication::clipboard();

						cb->setText( macro.mid(1) );
						QWSServer::sendKeyEvent('V'-'@',Qt::Key_V, Qt::ControlButton, true, false );
						QWSServer::sendKeyEvent('V'-'@',Qt::Key_V, Qt::ControlButton, false, false );

						c=0;
						m_lastDown1=m_lastDown2=0;
					}
				}else if((l>1)&&(macro[0]=='$')){
					if(down)
					{
						// Extension macro
						if(macro[1]=='$')
						{
							// escaped extension macro
							c='$';
						}else{
							// process the rest of the macro...
							macro=macro.mid(1);
							if(macro=="taskSwitch")
							{
								m_pTaskSwitcher->doTaskSwitch(true);
							}else if(macro=="taskSelector") {
								m_pTaskSelector->startSelector();
							}
							c=0;
							m_lastDown1=m_lastDown2=0;
						}
					}else{
						if(macro[1]=='$')
						{
							// escaped extension macro
							c='$';
						}else{
							if(macro=="taskSelector") 
							{
								m_pTaskSelector->endSelector();
							}
						
							c=0;
							m_lastDown1=m_lastDown2=0;
						}
					}
				}else if((l>1)&&(macro[0]=='+')){
					// Japanese special charactor macro
					if(macro[1]=='+')
					{
						// escaped macro
						c='+';
					}else{
						// convert numeric  from number to character...
						macro=macro.mid(1);
						c=macro.toUInt();
						unsigned short cg=0xFFFF;
						if(c>256)
						{
							cg=0xFFFF;
						}else if(c==0xa2){
							cg=0xFF62;
						}else if(c==0xa3){
							cg=0xFF63;
						}else if(c==0xa5){
							cg=0xFF65;
						}
						emit key(cg,c,m_lastMod,down,0);
						if(down)
						{
							// setup the key repeat
							startKeyRepeat(cg,c);
						}
						c=0;
					}
				}else{
					if(down)
					{
						int i=0,l=macro.length();
						QChar ch;
						if(l>1)
						{
							// macro is longer that a single key - emit it once, but don't key repeat
							while(i<l)
							{
								ch=macro[i++];
								emit key(ch,ch,m_lastMod,true,0);
								emit key(ch,ch,m_lastMod,false,0);
							}
							if((m_lastDown1==0xFFFF)&&(m_lastDown2==0xFFFF))
							{
								c=0;
								m_lastDown1=m_lastDown2=0;
							}
							c = 0;	// 2005.03.18 add
						}else{
							//  macro is single character - use normal keyboard handling to get key repeats and stuff...
							ch=macro[0];
							c=ch;
						}
					}
				}
			}
		}
	}
	
	if(c)
	{
		unsigned short c0,c1,m_lastModtmp;
		if(m_isVirtualfn&&(((c>='a')&&(c<='z'))|(c==',')|(c=='.')|(c=='-')|(c==' ')))
		{
			c0=m_fnjpc700unicode1LookupTable[c];
			if((c=='c')|(c=='v')|(c=='x')|(c=='z'))
				c1=Qt::ControlButton;
			else
				c1=0;
			c=m_fnjpc700scancodeLookupTable[c];
		}else{
			if( (m_lastMod&Qt::ControlButton) && ( ((c>='a')&&(c<='z')) || ((c>='A')&&(c<='Z')) ) )
			{
				c=c&~32;
				c0=c+1-'A';
				c1=m_lastMod;
			}else if((m_lastMod&Qt::ControlButton)&&((c==',')|(c=='.')|(c=='-')|(c==' '))){
				c0=m_fnjpc700unicode2LookupTable[c];
				c1=0;
				c=m_fnjpc700scancodeLookupTable[c];
			}else{
				c0=c;
				c1=m_lastMod;
			}

			if(c0>256)
				c0=0xFFFF;
		}
		if((c=='!')|(c=='#')|(c=='$')|(c=='%')|(c=='&')|(c=='(')|(c==')')|(c=='\'')){
			emit key(0xFFFF,Key_Shift,0,true,0);
			m_lastModtmp=m_lastMod;
			m_lastMod=Qt::ShiftButton;
			emit key(c0,c,m_lastMod,down,0);
			if(down)
			{
				// setup the key repeat
				startKeyRepeat(c0,c);
			}
			emit key(0xFFFF,Key_Shift,0,false,0);	
			m_lastMod=m_lastModtmp;
		}else if(c==':'){
			emit key(0xFFFF,Key_Shift,0,false,0);
			m_lastModtmp=m_lastMod;
			m_lastMod=0;
			emit key(c0,c,m_lastMod,down,0);
			if(down)
			{
				// setup the key repeat
				startKeyRepeat(c0,c);
			}
			emit key(0xFFFF,Key_Shift,0,true,0);
			m_lastMod=m_lastModtmp;
		}else{
			if((c>='a')&&(c<='z'))
				c^=32;
			emit key(c0,c,c1,down,0);
			// if(down&&!(c==0x104e)&&!(c==0x1009)&&!(c>=0x2007))
			if(down&!((c>=0x1030)&&(c<=0x1052))&!((c>=0x2007)&&(c<=0x2022)))
			{
				// setup the key repeat
				startKeyRepeat(c0,c);
			}
		}
	}

}


void CGIRK::dataAvailableLIRC(int fd)
{
	char buf[100],*parse;
	long l,i=0;
	bool isKeyUp=false;
	l=read(fd,buf,100);
	buf[l]='\0';
	if(l)
	{
		for(i=0; (i<l)&&(buf[i]!=' '); i++);
		if(buf[i]==' ')
		{
			for(i++; (i<l)&&(buf[i]!=' '); i++);
			if(buf[i]==' ')
			{
				i++;
				parse=&buf[i];
				for(i++; (i<l)&&(buf[i]!=' '); i++);
				if(buf[i]==' ')
				{
					buf[i]='\0';
					// translate named chars into proper chars
					if(!strcmp("space",parse)) 
					{
						parse[0]=' ';
						parse[1]='\0';
					}
					
					l=strlen(parse);
					
					if((parse[l-2]=='U')&&(parse[l-1]=='p'))
					{
						isKeyUp=true;
					}
					
					if(l==1)
					{
						handleKeyPress(parse[0],!isKeyUp);
					}
					
					// handle control keys
					if(!strncmp("shift",parse,5)) {
						if(isKeyUp)
						{
							emit key(0,Key_Shift,0,false,0);
							m_isShiftedl=m_isShiftedr=false;
						}else{
							emit key(0,Key_Shift,0,true,m_isShiftedl);
							m_shiftTimer->start(SHIFT_TIMEOUT,true);
							m_isShiftedl=m_isShiftedr=true;
						}
					}else if(!strcmp("pagehome",parse)) {
						emit key(0,Key_Home,0,true,0);
						emit key(0,Key_Home,0,false,0);
					}else if(!strcmp("pageend",parse)) {
						emit key(0,Key_End,0,true,0);
						emit key(0,Key_End,0,false,0);
					}else if(!strcmp("up",parse)) {
						emit key(0,Key_Up,0,true,0);
						emit key(0,Key_Up,0,false,0);
					}else if(!strcmp("down",parse)) {
						emit key(0,Key_Down,0,true,0);
						emit key(0,Key_Down,0,false,0);
					}else if(!strcmp("left",parse)) {
						emit key(0,Key_Left,0,true,0);
						emit key(0,Key_Left,0,false,0);
					}else if(!strcmp("right",parse)) {
						emit key(0,Key_Right,0,true,0);
						emit key(0,Key_Right,0,false,0);
					}else if(!strcmp("return",parse)) {
						emit key(0,Key_Return,0,true,0);
						emit key(0,Key_Return,0,false,0);
					}else if(!strcmp("pageup",parse)) {
						emit key(0,Key_PageUp,0,true,0);
						emit key(0,Key_PageUp,0,false,0);
					}else if(!strcmp("pagedown",parse)) {
						emit key(0,Key_PageDown,0,true,0);
						emit key(0,Key_PageDown,0,false,0);
					}else if(!strcmp("escape",parse)) {
						emit key(0,Key_Escape,0,true,0);
						emit key(0,Key_Escape,0,false,0);
					}else if(!strcmp("backspace",parse)) {
						emit key(0,Key_Backspace,0,true,0);
						emit key(0,Key_Backspace,0,false,0);
					}else if(!strcmp("tab",parse)) {
						emit key('\t',Key_Tab,0,true,0);
						emit key('\t',Key_Tab,0,false,0);
					}else if(!strcmp("caps",parse)) {
						emit key(0,Key_CapsLock,0,true,0);
						emit key(0,Key_CapsLock,0,false,0);
						// toggle the caps lock bit
						m_capsLock=!m_capsLock;
						m_caps->setChecked(m_capsLock);
					}else if(!strcmp("menu",parse)) {
						emit key(0,Key_Menu,0,true,0);
						emit key(0,Key_Menu,0,false,0);
					}else if(!strcmp("winl",parse)) {
						emit key(0,Key_Super_L,0,true,0);
						emit key(0,Key_Super_L,0,false,0);
					}else if(!strcmp("winr",parse)) {
						emit key(0,Key_Super_R,0,true,0);
						emit key(0,Key_Super_R,0,false,0);
					}else if(!strcmp("alt",parse)) {
						emit key(0,Key_Alt,0,true,0);
						emit key(0,Key_Alt,0,false,0);
					}else if(!strcmp("altgra",parse)) {
						emit key(0,Key_Hyper_R,0,true,0);
						emit key(0,Key_Hyper_R,0,false,0);
					}
				}
				
			}
		}
	}
}

unsigned short CGIRK::modifiers()
{
	unsigned short rtn=0;
	if(m_isCtrl)
		rtn|=Qt::ControlButton;
	if(m_isShiftedl|m_isShiftedr)
		rtn|=Qt::ShiftButton;
	if(m_isAlt)
		rtn|=Qt::AltButton;
	return rtn;
}

void CGIRK::dataAvailablePatapata(int fd)
{
	int l;
	unsigned char c;
	l=read(fd,&c,1);
	if(l==1)
	{
		bool down=(c&0x80)?false:true;
		unsigned short code;
		code=m_pocketopLookupTable[c&0x7f];
		
		if(m_rawMode)
		{
			emit key((isprint(code)?code:0xFFFF),code,m_lastMod,down,0);
			return;
		}
		
		if(!down)
			m_repeatTimer->stop();

		if(code)
		{
			handleKeyPress(code,down);
		}else{
#if DEBUG_KEY_CODES
			QString msg;
			msg.sprintf("Unrecognised keycode 0x%x",c);
			QMessageBox mb("IRK",msg,QMessageBox::NoIcon,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
			mb.setIconPixmap(QPixmap((const char **)targus_xpm));
			mb.exec();
#endif

		}
	}
}

void CGIRK::dataAvailableTKB780U(int fd)
{
	int l;
	static unsigned char lastscan;
	static bool lastdown;
	static bool modifieron=false;
	static unsigned short lastcode;
	unsigned char c;
	l=read(fd,&c,1);
	if(l==1)
	{
		bool down=(c&0x80)?true:false;
		unsigned short code;
		code=m_pocketopLookupTable[c&0x7f];
		
		if(m_rawMode)
		{
			emit key((isprint(code)?code:0xFFFF),code,m_lastMod,down,0);
			return;
		}
		
		if ((lastscan==0x02)|!down|modifieron)
		{
			if(!down)
				m_repeatTimer->stop();

			if(code)
			{
				if(!((lastcode==code)&&(lastdown==down))){
					handleKeyPress(code,down);
					if(down)
						modifieron=true;
				}else{
					m_isVirtualfn=false;
					modifieron=false;
				}
				lastcode=code;
				lastdown=down;
			}else{
#if DEBUG_KEY_CODES
				QString msg;
				msg.sprintf("Unrecognised keycode 0x%x",c);
				QMessageBox mb("IRK",msg,QMessageBox::NoIcon,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
				mb.setIconPixmap(QPixmap((const char **)palm_xpm));
				mb.exec();
#endif
			}
		}
		lastscan=c;
	}
}

void CGIRK::dataAvailableF8U1500(int fd)
{
	int l;
	static unsigned char lastscan;
	unsigned char c;
	l=read(fd,&c,1);
	if(l==1)
	{
		bool down=(c&0x80)?false:true;
		unsigned char d=c%0x10;
		unsigned short code;
		code=m_pocketopLookupTable[c&0x7f];
		if(m_rawMode)
		{
			if((down&&!(d==2))|((c==lastscan)&&!down)|((lastscan==0x02)&&(c==0x12))|((lastscan==0x42)&&(c==0x42))|((lastscan==0x62)&&(c==0x72))|((lastscan==0xa2)&&(c==0x32))|((lastscan==0xc2)&&(c==0x52)))
				emit key((isprint(code)?code:0xFFFF),code,m_lastMod,down,0);
			lastscan=c;
			return;
		}
		
		if((down&&!(d==2))|((c==lastscan)&&!down)|((lastscan==0x02)&&(c==0x12))|((lastscan==0x42)&&(c==0x42))|((lastscan==0x62)&&(c==0x72))|((lastscan==0xa2)&&(c==0x32))|((lastscan==0xc2)&&(c==0x52)))
		{
			if(!down)
				m_repeatTimer->stop();

			if(code)
			{
				handleKeyPress(code,down);
				if(!down&&(code==CGIRK_KEY_CMD|code==CGIRK_KEY_SHIFTL|code==CGIRK_KEY_SHIFTR))
					m_isVirtualfn=false;
			}else{
#if DEBUG_KEY_CODES
				QString msg;
				msg.sprintf("Unrecognised keycode 0x%x",c);
				QMessageBox mb("IRK",msg,QMessageBox::NoIcon,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
				mb.setIconPixmap(QPixmap((const char **)palm_xpm));
				mb.exec();
#endif
			}
		}
		lastscan=c;
	}
}

void CGIRK::dataAvailableF8U1500T(int fd)
{
	int l;
	static bool lastdown;
	static unsigned char lastscan;
	static unsigned short lastcode;
	unsigned char c;
	l=read(fd,&c,1);
	if(l==1)
	{
		bool down=(c&0x80)?false:true;
		unsigned short code;
		code=m_pocketopLookupTable[c&0x7f];
		
		if(m_rawMode)
		{
			if(!(lastscan==0xc0))
				emit key((isprint(code)?code:0xFFFF),code,m_lastMod,down,0);
			lastscan=c;
			return;
		}
		
		if (c==0xc1)
		{
			if(!lastdown)
				m_repeatTimer->stop();

			if(lastcode)
			{
				handleKeyPress(lastcode,lastdown);
				if(!lastdown&&(lastcode==CGIRK_KEY_CMD|lastcode==CGIRK_KEY_SHIFTL|lastcode==CGIRK_KEY_SHIFTR))
					m_isVirtualfn=false;
			}else{
#if DEBUG_KEY_CODES
				QString msg;
				msg.sprintf("Unrecognised keycode 0x%x",c);
				QMessageBox mb("IRK",msg,QMessageBox::NoIcon,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
				mb.setIconPixmap(QPixmap((const char **)palm_xpm));
				mb.exec();
#endif
			}
		}
		lastcode=code;
		lastdown=down;
	}
}

void CGIRK::dataAvailablePalmOne3169WW(int fd)
{
	int l;
	static unsigned char lastscan;
	static bool lastdown;
	static unsigned short lastcode;
	unsigned char c;
	l=read(fd,&c,1);
	if(l==1)
	{
		bool down=(c&0x80)?false:true;
		unsigned short code;
		code=m_pocketopLookupTable[c&0x7f];
		
		if(m_rawMode)
		{
			if(!(lastscan==0xc0))
				emit key((isprint(code)?code:0xFFFF),code,m_lastMod,down,0);
			return;
		}
		
		if (lastscan==0xc0)
		{
			if(!down)
				m_repeatTimer->stop();

			if(code)
			{
				if(!((lastcode==code)&&(lastdown==down)))
					handleKeyPress(code,down);
				else
					m_isVirtualfn=false;
				lastcode=code;
				lastdown=down;
			}else{
#if DEBUG_KEY_CODES
				QString msg;
				msg.sprintf("Unrecognised keycode 0x%x",c);
				QMessageBox mb("IRK",msg,QMessageBox::NoIcon,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
				mb.setIconPixmap(QPixmap((const char **)palm_xpm));
				mb.exec();
#endif
			}
		}
		lastscan=c;
	}
}

void CGIRK::dataAvailableHPfoldableBT(int fd)
{
	int l;
	static bool lastdown;
	static unsigned short lastcode;
	unsigned char c;
	l=read(fd,&c,1);
	if(l==1)
	{
		bool down=(c&0x80)?false:true;
		unsigned short code;
		code=m_pocketopLookupTable[c&0x7f];
		
		if(m_rawMode)
		{
			emit key((isprint(code)?code:0xFFFF),code,m_lastMod,down,0);
			return;
		}
		
		if(!down)
			m_repeatTimer->stop();

		if(code)
		{
			if(!((lastcode==code)&&(lastdown==down)))
				handleKeyPress(code,down);
			else
				m_isVirtualfn=false;
			lastcode=code;
			lastdown=down;
		}else{
#if DEBUG_KEY_CODES
			QString msg;
			msg.sprintf("Unrecognised keycode 0x%x",c);
			QMessageBox mb("IRK",msg,QMessageBox::NoIcon,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
			mb.setIconPixmap(QPixmap((const char **)pocketop_xpm));
			mb.exec();
#endif
		}
	}
}

void CGIRK::dataAvailablePalm(int fd)
{
	int l;
	static unsigned char lastscan;
	static bool lastdown;
	static unsigned short lastcode;
	unsigned char c;
	l=read(fd,&c,1);
	if(l==1)
	{
		bool down=(c&0x80)?false:true;
		unsigned short code;
		code=m_pocketopLookupTable[c&0x7f];
		
		if(m_rawMode)
		{
			emit key((isprint(code)?code:0xFFFF),code,m_lastMod,down,0);
			return;
		}
		
		if ((lastscan==0xc0)&&(c!=0x5c))
		{
			if(!down)
				m_repeatTimer->stop();

			if(code)
			{
				if(!((lastcode==code)&&(lastdown==down)))
					handleKeyPress(code,down);
				else
					m_isVirtualfn=false;
				lastcode=code;
				lastdown=down;
			}else{
#if DEBUG_KEY_CODES
				QString msg;
				msg.sprintf("Unrecognised keycode 0x%x",c);
				QMessageBox mb("IRK",msg,QMessageBox::NoIcon,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
				mb.setIconPixmap(QPixmap((const char **)palm_xpm));
				mb.exec();
#endif
			}
		}
		lastscan=c;
	}
}

void CGIRK::dataAvailableUS102(int fd)
{
	int l;

	static unsigned short lastcode = 0;
	static unsigned short lastC = 0;
/*
	static bool lastdown;
	static unsigned short lastcode;
*/
	unsigned char c;
	l=read(fd,&c,1);
	if(l==1)
	{
		if (c != 0xf0) {
			unsigned short code;
			code=m_pocketopLookupTable[c&0x7f];
			
			if(m_rawMode)
			{
				emit key((isprint(code)?code:0xFFFF),code,m_lastMod,lastC != 0xf0,0);
				return;
			}
			
			if (lastC == 0xf0) {
				if (lastcode == code)
					m_repeatTimer->stop();
				else
					m_isVirtualfn=false;
			}
			if(code)
			{
				handleKeyPress(code,lastC != 0xf0);
			}
			lastcode=code;
		}else{
#if DEBUG_KEY_CODES
			QString msg;
			msg.sprintf("Unrecognised keycode 0x%x",c);
			QMessageBox mb("IRK",msg,QMessageBox::NoIcon,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
			mb.setIconPixmap(QPixmap((const char **)pocketop_xpm));
			mb.exec();
#endif
		}
	}
	lastC = c;
}

void CGIRK::dataAvailableJP106(int fd)
{
	int l;

	static unsigned short lastcode = 0;
	static unsigned short lastC = 0;
/*
	static bool lastdown;
	static unsigned short lastcode;
*/
	unsigned char c;
	l=read(fd,&c,1);
	if(l==1)
	{
		if (c == 0xe0) {
		  return;
		}
		if (c != 0xf0) {
			unsigned short code;
			code=m_pocketopLookupTable[c&0x7f];
			
			if(m_rawMode)
			{
				emit key((isprint(code)?code:0xFFFF),code,m_lastMod,lastC != 0xf0,0);
				return;
			}
			
			if(c == 0x0e)
			{
				emit key(0xFFFF,Key_F25,m_lastMod,lastC != 0xf0,0);
				lastcode=code;
				lastC = c;
				return;
			}
			
			if (lastC == 0xf0) {
				if (lastcode == code)
					m_repeatTimer->stop();
				else
					m_isVirtualfn=false;
			}
			if(code)
			{
				handleKeyPress(code,lastC != 0xf0);
			}
			lastcode=code;
		}else{
#if DEBUG_KEY_CODES
			QString msg;
			msg.sprintf("Unrecognised keycode 0x%x",c);
			QMessageBox mb("IRK",msg,QMessageBox::NoIcon,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
			mb.setIconPixmap(QPixmap((const char **)pocketop_xpm));
			mb.exec();
#endif
		}
	}
	lastC = c;
}

void CGIRK::dataAvailableTargus(int fd)
{
	int l;
	unsigned char c;
	l=read(fd,&c,1);
	if(l==1)
	{
		bool down=(c&0x80)?false:true;
		unsigned short code;
		code=m_pocketopLookupTable[c&0x7f];
		
		if(m_rawMode)
		{
			emit key((isprint(code)?code:0xFFFF),code,m_lastMod,down,0);
			return;
		}
		
		if(!down)
			m_repeatTimer->stop();

		if(code)
		{
			handleKeyPress(code,down);
		}else{
#if DEBUG_KEY_CODES
			QString msg;
			msg.sprintf("Unrecognised keycode 0x%x",c);
			QMessageBox mb("IRK",msg,QMessageBox::NoIcon,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
			mb.setIconPixmap(QPixmap((const char **)targus_xpm));
			mb.exec();
#endif

		}
	}
}

void CGIRK::dataAvailablePocketop(int fd)
{
	int l;
	static bool lastdown;
	static unsigned short lastcode;
	unsigned char c;
	l=read(fd,&c,1);
	if(l==1)
	{
		bool down=(c&0x80)?false:true;
		unsigned short code;
		code=m_pocketopLookupTable[c&0x7f];
		
		if(m_rawMode)
		{
			emit key((isprint(code)?code:0xFFFF),code,m_lastMod,down,0);
			return;
		}
		
		if(!down)
			m_repeatTimer->stop();

		if(code)
		{
			if(!((lastcode==code)&&(lastdown==down)))
				handleKeyPress(code,down);
			else
				m_isVirtualfn=false;
			lastcode=code;
			lastdown=down;
		}else{
#if DEBUG_KEY_CODES
			QString msg;
			msg.sprintf("Unrecognised keycode 0x%x",c);
			QMessageBox mb("IRK",msg,QMessageBox::NoIcon,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
			mb.setIconPixmap(QPixmap((const char **)pocketop_xpm));
			mb.exec();
#endif
		}
	}
}

void CGIRK::repeat()
{
	if(m_lastDown1)
	{
		m_repeatTimer->start(m_keyRepeatPeriod);
		emit key(m_lastDown1,m_lastDown2,m_lastMod,true,true);
	}else{
		m_repeatTimer->stop();
	}
}

void CGIRK::setRaw(bool f)
{
	m_rawMode=f;
}

bool CGIRK::raw()
{
	return m_rawMode;
}

void CGIRK::startKeyRepeat(unsigned short c0, unsigned short c1)
{
	m_lastDown1=c0; m_lastDown2=c1;
	if(m_keyRepeatEnable)
		m_repeatTimer->start(m_keyRepeatDelay);
}


QChar CGIRK::specialForName(const QString &name)
{
	QChar ch=0;
	if(name.length()>1)
	{
		if(name=="CAPS")
			ch=CGIRK_KEY_CAPS;
		else if(name=="SHIFTL")
			ch=CGIRK_KEY_SHIFTL;
		else if(name=="SHIFTR")
			ch=CGIRK_KEY_SHIFTR;
		else if(name=="TAB")
			ch=CGIRK_KEY_TAB;
		else if(name=="CTRL")
			ch=CGIRK_KEY_CTRL;
		else if(name=="PUNCT")
			ch=CGIRK_KEY_PUNCT;
		else if(name=="CMD")
			ch=CGIRK_KEY_CMD;
		else if(name=="FNL")
			ch=CGIRK_KEY_FNL;
		else if(name=="FNR")
			ch=CGIRK_KEY_FNR;
		else if(name=="NUM")
			ch=CGIRK_KEY_NUM;
		else if(name=="ALT")
			ch=CGIRK_KEY_ALT;
		else if(name=="FUNCTION")
			ch=CGIRK_KEY_FUNCTION;
		else if(name=="LEFT")
			ch=CGIRK_KEY_LEFT;
		else if(name=="RIGHT")
			ch=CGIRK_KEY_RIGHT;
		else if(name=="UP")
			ch=CGIRK_KEY_UP;
		else if(name=="DOWN")
			ch=CGIRK_KEY_DOWN;
		else if(name=="DEL")
			ch=CGIRK_KEY_DEL;
		else if(name=="TAB")
			ch=CGIRK_KEY_TAB;
		else if(name=="BACKSPACE")
			ch=CGIRK_KEY_BACKSPACE;
		else if(name=="RETURN")
			ch=CGIRK_KEY_RETURN;
		else if(name=="DATE")
			ch=CGIRK_KEY_DATE;
		else if(name=="ADDRESS")
			ch=CGIRK_KEY_ADDRESS;
		else if(name=="TODO")
			ch=CGIRK_KEY_TODO;
		else if(name=="MEMO")
			ch=CGIRK_KEY_MEMO;
	}else{
		ch=name[0];
	}
	return ch;

}

const char *CGIRK::nameForSpecial(int n)
{
	switch(n)
	{
		case CGIRK_KEY_CAPS:
			return "CAPS";
			break;
		case CGIRK_KEY_SHIFTL:
			return "SHIFTL";
			break;
		case CGIRK_KEY_SHIFTR:
			return "SHIFTR";
			break;
		case CGIRK_KEY_CTRL:
			return "CTRL";
			break;
		case CGIRK_KEY_PUNCT:
			return "PUNCT";
			break;
		case CGIRK_KEY_CMD:
			return "CMD";
			break;
		case CGIRK_KEY_FNL:
			return "FNL";
			break;
		case CGIRK_KEY_FNR:
			return "FNR";
			break;
		case CGIRK_KEY_NUM:
			return "NUM";
			break;
		case CGIRK_KEY_ALT:
			return "ALT";
			break;
		case CGIRK_KEY_FUNCTION:
			return "FUNCTION";
			break;
		case CGIRK_KEY_LEFT:
			return "LEFT";
			break;
		case CGIRK_KEY_RIGHT:
			return "RIGHT";
			break;
		case CGIRK_KEY_UP:
			return "UP";
			break;
		case CGIRK_KEY_DOWN:
			return "DOWN";
			break;
		case CGIRK_KEY_DEL:
			return "DEL";
			break;
		case CGIRK_KEY_TAB:
			return "TAB";
			break;
		case CGIRK_KEY_BACKSPACE:
			return "BACKSPACE";
			break;
		case CGIRK_KEY_RETURN:
			return "RETURN";
			break;
		case CGIRK_KEY_DATE:
			return "DATE";
			break;
		case CGIRK_KEY_ADDRESS:
			return "ADDRESS";
			break;
		case CGIRK_KEY_TODO:
			return "TODO";
			break;
		case CGIRK_KEY_MEMO:
			return "MEMO";
			break;
		default:
			return "???";
			break;
	}	
}

CIRKeyboard::CIRKeyboard(QString name)
{
	m_name=name;
/*	memset(m_shiftMap,0,sizeof(m_shiftMap));
	memset(m_cmdMap,0,sizeof(m_cmdMap));
	memset(m_fnlMap,0,sizeof(m_fnlMap));
	memset(m_fnrMap,0,sizeof(m_fnrMap));
	memset(m_numMap,0,sizeof(m_numMap));
	memset(m_punctMap,0,sizeof(m_punctMap));*/
}

CIRKeyboard::~CIRKeyboard()
{
}

void CIRKeyboard::setMode(IRKMODE mode)
{
	m_mode=mode;
}

void CIRKeyboard::setShiftMapEntry(unsigned char key,QString shifted)
{
	m_shiftMap[key]=shifted;
}

void CIRKeyboard::setPunctMapEntry(unsigned char key,QString shifted)
{
	m_punctMap[key]=shifted;
}

void CIRKeyboard::setCmdMapEntry(unsigned char key,QString shifted)
{
	m_cmdMap[key]=shifted;
}

void CIRKeyboard::setFnlMapEntry(unsigned char key,QString shifted)
{
	m_fnlMap[key]=shifted;
}

void CIRKeyboard::setFnrMapEntry(unsigned char key,QString shifted)
{
	m_fnrMap[key]=shifted;
}

void CIRKeyboard::setNumMapEntry(unsigned char key,QString shifted)
{
	m_numMap[key]=shifted;
}

void CIRKeyboard::setFastMapEntry(unsigned char key,QString shifted)
{
	m_fastMap[key]=shifted;
}

void CIRKeyboard::setLIRCConf(QString conf)
{
	m_lircConf=conf;
}

void CIRKeyboard::setNumShifted(int n)
{
	m_numShifted=n;
}

void CIRKeyboard::setNumNum(int n)
{
	m_numNum=n;
}

void CIRKeyboard::setNumCmd(int n)
{
	m_numCmd=n;
}

void CIRKeyboard::setNumFnl(int n)
{
	m_numFnl=n;
}

void CIRKeyboard::setNumFnr(int n)
{
	m_numFnr=n;
}

void CIRKeyboard::setNumPunct(int n)
{
	m_numPunct=n;
}

void CIRKeyboard::setNumFast(int n)
{
	m_numFast=n;
}


QString CIRKeyboard::name()
{
	return m_name;
}

QString CIRKeyboard::lircConf()
{
	return m_lircConf;
}

IRKMODE CIRKeyboard::mode()
{
	return m_mode;
}

QString CIRKeyboard::shiftMapEntry(unsigned char  c)
{
	return m_shiftMap[c];
}

QString CIRKeyboard::numMapEntry(unsigned char  c)
{
	return m_numMap[c];
}

QString CIRKeyboard::cmdMapEntry(unsigned char  c)
{
	return m_cmdMap[c];
}

QString CIRKeyboard::fnlMapEntry(unsigned char  c)
{
	return m_fnlMap[c];
}

QString CIRKeyboard::fnrMapEntry(unsigned char  c)
{
	return m_fnrMap[c];
}

QString CIRKeyboard::punctMapEntry(unsigned char  c)
{
	return m_punctMap[c];
}

QString CIRKeyboard::fastMapEntry(unsigned char  c)
{
	return m_fastMap[c];
}

QString *CIRKeyboard::shiftMapArray()
{
	return m_shiftMap;
}

QString *CIRKeyboard::numMapArray()
{
	return m_numMap;
}

QString *CIRKeyboard::cmdMapArray()
{
	return m_cmdMap;
}

QString *CIRKeyboard::fnlMapArray()
{
	return m_fnlMap;
}

QString *CIRKeyboard::fnrMapArray()
{
	return m_fnrMap;
}

QString *CIRKeyboard::punctMapArray()
{
	return m_punctMap;
}

QString *CIRKeyboard::fastMapArray()
{
	return m_fastMap;
}

int CIRKeyboard::numShifted()
{
	return m_numShifted;
}

int CIRKeyboard::numPunct()
{
	return m_numPunct;
}

int CIRKeyboard::numCmd()
{
	return m_numCmd;
}

int CIRKeyboard::numFnl()
{
	return m_numFnl;
}

int CIRKeyboard::numFnr()
{
	return m_numFnr;
}

int CIRKeyboard::numNum()
{
	return m_numNum;
}

int CIRKeyboard::numFast()
{
	return m_numFast;
}

void CIRKeyboard::resetMaps()
{
	int i; 
	for(i=0; i<256; i++)
	{
		m_cmdMap[i]="";
		m_fnlMap[i]="";
		m_fnrMap[i]="";
		m_numMap[i]="";
		m_shiftMap[i]="";
		m_punctMap[i]="";
		m_fastMap[i]="";
	}
	m_numFast=
	m_numNum=
	m_numCmd=
	m_numFnl=
	m_numFnr=
	m_numShifted=
	m_numPunct=0;
}

