#ifndef _IRK_H_
#define _IRK_H_

#include <qvariant.h>
#include <qframe.h>
#include <qstring.h>
#include <qlist.h>
#include "cgirkbase.h"
#include "TaskSelector.h"
#include "TaskSwitcher.h"

class QSocketNotifier;
class QRadioButton;
class QTimer;
class QCheckBox;
class QPushButton;
class CIRKSettingsForm;

#define SHIFT_TIMEOUT 30000
typedef enum { IRKMODE_LIRC=0, IRKMODE_IRDA=1, IRKMODE_POCKETOP=2, IRKMODE_TARGUS=3, IRKMODE_PALM=4, IRKMODE_TKB780U=5, IRKMODE_F8U1500=6, IRKMODE_F8U1500T=7,IRKMODE_PalmOne3169WW=8, IRKMODE_HPfoldableBT=9, IRKMODE_PATAPATA=10, IRKMODE_US102=11, IRKMODE_JP106=12 } IRKMODE;
typedef enum { SL5500=0, SL5600=1, BTrfcomm0=2, BTrfcomm1=3, SLSERIAL=4, ZAURUS_MODEL_MAX } ZAURUS_MODEL;

#define CGIRK_KEY_CAPS         1
#define CGIRK_KEY_SHIFTL       2
#define CGIRK_KEY_SHIFTR       3
#define CGIRK_KEY_CTRL         4
#define CGIRK_KEY_PUNCT        5
#define CGIRK_KEY_CMD          6
#define CGIRK_KEY_FNL          7
#define CGIRK_KEY_FNR          8
#define CGIRK_KEY_NUM          14
#define CGIRK_KEY_ALT          15
#define CGIRK_KEY_FUNCTION     16
#define CGIRK_KEY_LEFT         17
#define CGIRK_KEY_RIGHT        18
#define CGIRK_KEY_UP           19
#define CGIRK_KEY_DOWN         20
#define CGIRK_KEY_DEL          21
#define CGIRK_KEY_TAB          22
#define CGIRK_KEY_BACKSPACE    23
#define CGIRK_KEY_RETURN       24
#define CGIRK_KEY_DATE         25
#define CGIRK_KEY_ADDRESS      26
#define CGIRK_KEY_TODO         27
#define CGIRK_KEY_MEMO         28

#define CGIRK_KEY_ESC          29
#define CGIRK_KEY_CONFIRM      30
#define CGIRK_KEY_HOME         31

class CIRKeyboard {
public:
	CIRKeyboard(QString name);
	~CIRKeyboard();
	void setMode(IRKMODE mode);
	void setLIRCConf(QString conf);
	
	void setShiftMapEntry(unsigned char key,QString shifted);
	void setNumShifted(int n);
	void setCmdMapEntry(unsigned char key,QString shifted);
	void setNumCmd(int n);
	void setFnlMapEntry(unsigned char key,QString shifted);
	void setNumFnl(int n);
	void setFnrMapEntry(unsigned char key,QString shifted);
	void setNumFnr(int n);
	void setPunctMapEntry(unsigned char key,QString shifted);
	void setNumPunct(int n);
	void setNumMapEntry(unsigned char key,QString shifted);
	void setNumNum(int n);
	void setFastMapEntry(unsigned char key,QString shifted);
	void setNumFast(int n);
	
	QString name();
	QString lircConf();
	IRKMODE mode();
	
	QString shiftMapEntry(unsigned char  c);
	QString numMapEntry(unsigned char  c);
	QString punctMapEntry(unsigned char  c);
	QString cmdMapEntry(unsigned char  c);
	QString fnlMapEntry(unsigned char  c);
	QString fnrMapEntry(unsigned char  c);
	QString fastMapEntry(unsigned char  c);
	QString *shiftMapArray();
	QString *numMapArray();
	QString *punctMapArray();
	QString *cmdMapArray();
	QString *fnlMapArray();
	QString *fnrMapArray();
	QString *fastMapArray();
	
	void resetMaps();
	
	int numShifted();
	int numNum();
	int numPunct();
	int numCmd();
	int numFnl();
	int numFnr();
	int numFast();
private:
	QString m_name;
	QString m_lircConf;
	IRKMODE m_mode;
	int m_numShifted;
	QString m_shiftMap[256];
	int m_numNum;
	QString m_numMap[256];
	int m_numPunct;
	QString m_punctMap[256];
	int m_numCmd;
	QString m_cmdMap[256];
	int m_numFnl;
	QString m_fnlMap[256];
	int m_numFnr;
	QString m_fnrMap[256];
	int m_numFast;
	QString m_fastMap[256];
};

class CGIRK : public CGIRKBase
{
    Q_OBJECT
    
public:
	CGIRK(QWidget * parent=0, const char * name=0, WFlags f=0);
	~CGIRK();
	
	virtual QSize sizeHint() const;
//	virtual void resizeEvent(QResizeEvent*);

	CIRKeyboard *currentKeyboard(int n=-1);
	
	void setRepeat(int ms,int delay=-1);
	
	/** Set raw mode (no caps handling, etc) 
	 * @param f true=on, false=off
	 */
	void setRaw(bool f);
	/** Check raw mode state */
	bool raw();
	
	/** Get name for special key */
	static const char *nameForSpecial(int n);
	static QChar specialForName(const QString &name);

signals:
	void key( ushort scancode, ushort unicode, ushort modifiers, bool, bool );
	
public slots:
	void dataAvailableLIRC(int socket);
	void dataAvailablePocketop(int socket);
	void dataAvailableTargus(int socket);
	void dataAvailablePalm(int socket);
	void dataAvailableTKB780U(int socket);
	void dataAvailableF8U1500(int socket);
	void dataAvailableF8U1500T(int socket);
	void dataAvailablePalmOne3169WW(int socket);
	void dataAvailableHPfoldableBT(int socket);
	void dataAvailablePatapata(int socket);
	void dataAvailableUS102(int socket);
	void dataAvailableJP106(int socket);
	void on(bool state);
	void off(bool state );
	void caps(bool state );
	void num(bool state );
	void cancelShift();
	void settings();
	void repeat();
	
private:
	void setStatus(const char *s);
	void handleKeyPress(unsigned short c,bool down);
	void startKeyRepeat(unsigned short c0, unsigned short c1);
	
	QString getShifted(char c);
	QString getNum(char c);
	QString getPunct(char c);
	QString getCmd(char c);
	QString getFnl(char c);
	QString getFnr(char c);
	QString getFast(char c);
	
	void readConfig();
	void writeConfig();
	void keyboardOpen();
	void keyboardClose();
	void lircOpen();
	void lircClose();
	
	void pocketopOpen();
	void pocketopClose();
	void targusOpen();
	void targusClose();
	void palmOpen();
	void palmClose();
	void TKB780UOpen();
	void TKB780UClose();
	void F8U1500Open();
	void F8U1500Close();
	void F8U1500TOpen();
	void F8U1500TClose();
	void PalmOne3169WWOpen();
	void PalmOne3169WWClose();
	void HPfoldableBTOpen();
	void HPfoldableBTClose();
	void patapataOpen();
	void patapataClose();
	void US102Open();
	void US102Close();
	void JP106Open();
	void JP106Close();
	/** init the pocketop keycode translation table */
	void c700Init();
	void pocketopInit();
	void pocketopfeedBits(unsigned int count,int thisBitLevel,int timeout);
	void targusInit();
	void targusfeedBits(unsigned int count,int thisBitLevel,int timeout);
	void palmInit();
	void palmfeedBits(unsigned int count,int thisBitLevel,int timeout);
	void TKB780UInit();
	void TKB780UfeedBits(unsigned int count,int thisBitLevel,int timeout);
	void F8U1500Init();
	void F8U1500feedBits(unsigned int count,int thisBitLevel,int timeout);
	void F8U1500TInit();
	void F8U1500TfeedBits(unsigned int count,int thisBitLevel,int timeout);
	void PalmOne3169WWInit();
	void PalmOne3169WWfeedBits(unsigned int count,int thisBitLevel,int timeout);
	void HPfoldableBTInit();
	void HPfoldableBTfeedBits(unsigned int count,int thisBitLevel,int timeout);
	void patapataInit();
	void patapatafeedBits(unsigned int count,int thisBitLevel,int timeout);
	
	void US102Init();
	void US102feedBits(unsigned int count,int thisBitLevel,int timeout);

	void JP106Init();
	void JP106feedBits(unsigned int count,int thisBitLevel,int timeout);

	unsigned short modifiers();
	
	bool isNum();
	
	// Configuration stuff
	/** current comms mode */
	CIRKeyboard *m_currentKeyboard;

	int m_keyRepeatPeriod;
	int m_keyRepeatDelay;
	bool m_keyRepeatEnable;
	bool m_swapCtrlCaps;
		
	/** List of available keyboards */
	QList<CIRKeyboard> m_keyboardList;
	
	int fd;
	QSocketNotifier *sn;
	QTimer *m_shiftTimer;
	QTimer *m_repeatTimer;
	CIRKSettingsForm *pSettingsForm;
	TaskSelector *m_pTaskSelector;
	TaskSwitcher *m_pTaskSwitcher;
	/** Shift key is currently down (activated when Shift is recieved, 
	    cleared when ShiftUp recieved, or when the shift timer expires 
	    (in case the keyup message goes missing) */
	bool m_isShiftedl;
	bool m_isShiftedr;
	/** Punct key is currently down */
	bool m_isPunct;
	/** Cmd key is currently down */
	bool m_isCmd;
	/** Ctrl key is currently down */
	bool m_isFnl;
	/** Fn(L) key is currently down */
	bool m_isFnr;
	/** Fn(R) key is currently down */
	bool m_isCtrl;
	/** Num key is currently down */
	bool m_isNum;
	/** Cmd+shiftr/shiftl keys are currently down */
	bool m_isVirtualfn;
	/** Alt key is currently down */
	bool m_isAlt;
	/** Caps lock has been activated (this toggles whenever caps is pressed) */
	bool m_capsLock;
	/** Num lock has been activated */
	bool m_numLock;
	/** Keyboard is active flag */
	bool m_keyboardOpen;
	bool m_pocketopStarted;
	bool m_pocketopHadStartBit;
	/** last down keypress (or 0 for none) */
	unsigned short m_lastDown1,m_lastDown2;
	/** modified for the last down keypress */
	unsigned short m_lastMod;
	/** Pocketop 'byte', used for key signature generation */
	unsigned short m_byte;
	/** Pocketop bit number, used for key signature generation */
	unsigned int m_bit;
	
	/** rotation */
	unsigned short m_rotation;
	bool m_isRotated;
	/** light control */
	int m_lightlevel;
	/** light control */
	bool m_lighton;
	/** raw mode (don't process macro's and stuff) */
	bool m_rawMode;
	/** Pocketop keycode lookup table... */
	static unsigned short m_pocketopLookupTable[128];
	static unsigned short m_fnjpc700scancodeLookupTable[128];
	static unsigned short m_fnjpc700unicode1LookupTable[128];
	static unsigned short m_fnjpc700unicode2LookupTable[128];
	/** Which zaurus are we running on? */
	ZAURUS_MODEL m_zaurusModel;
	/** IrDA device names for various Zaurii */
	static const char *m_irdaDeviceNames[ZAURUS_MODEL_MAX];
};

#endif
